(* Delphi dialog
   Edit password with confirmation

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - June 2005
   *)
   
unit AccountDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, JrUtils;

const
  defMinPwdLength = 6;

type
  TAccountDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    lblRepPwd: TLabel;
    edtRepPwd: TEdit;
    Label1: TLabel;
    edtPwd: TEdit;
    btnShow: TSpeedButton;
    Label2: TLabel;
    edtName: TEdit;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FConf   : boolean;
    FMinLen : integer;
  public
    { Public declarations }
    function Execute (Title : string; var Username,Pwd : string;
                      MinLen   : integer; Confirm,Show : boolean) : boolean;
  end;

var
  AccountDialog: TAccountDialog;

implementation

{$R *.DFM}

uses GnuGetText;

procedure TAccountDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TAccountDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) and FConf then
    CanClose:=(edtPwd.Text=edtRepPwd.Text) and (length(edtPwd.Text)>=FMinLen);
  if not CanClose then begin
    if length(edtPwd.Text)<FMinLen then begin
      ErrorDialog(Caption,Format(dgettext('dialogs','Minimum length of password is %u character'),[FMinLen]),CenterPos);
      edtPwd.SetFocus;
      end
    else begin
      ErrorDialog(Caption,dgettext('dialogs','Passwords do not match!'),CenterPos);
      edtRepPwd.SetFocus;
      end;
    end;
  end;

procedure TAccountDialog.btnShowClick(Sender: TObject);
var
  c : char;
begin
  if btnShow.Down then c:=#0 else c:='*';
  edtPwd.PasswordChar:=c;
  edtRepPwd.PasswordChar:=c;
  end;

{ ------------------------------------------------------------------- }
(* Passworteingabe, Ergebnis: "true" bei "ok" *)
function TAccountDialog.Execute (Title    : string;
                                var Username,Pwd  : string;
                                MinLen   : integer;
                                Confirm,Show  : boolean) : boolean;
begin
  Caption:=Title;
  edtName.Text:=Username;
  edtPwd.Text:=Pwd;
  edtRepPwd.Text:=Pwd;
  FConf:=Confirm;
  FMinLen:=MinLen;
  btnShow.Visible:=Show;
  if FConf then begin
    lblRepPwd.Show; edtRepPwd.Show;
    end
  else begin
    lblRepPwd.Hide; edtRepPwd.Hide;
    end;
  ActiveControl:=edtName;
  if ShowModal=mrOK then begin
    Username:=edtName.Text;
    Pwd:=edtPwd.Text;
    Result:=true;
    end
  else Result:=false;
  end;

end.
