(*  Delphi Dialog
    Zeichentabelle (Character table)
    ================================
    
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Sep. 2002 
   *)
    
unit CharTableDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, JrUtils;

type
  TCharTableDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    CharGrid: TDrawGrid;
    procedure FormCreate(Sender: TObject);
    procedure CharGridDblClick(Sender: TObject);
    procedure CharGridDrawCell(Sender: TObject; ACol, ARow: Integer;
      Rect: TRect; State: TGridDrawState);
  private
    { Private declarations }
    DefFontName,GridFontName : TFontName;
  public
    { Public declarations }
  end;

function CharFromtable (AFontName : TFontName) : char;

var
  CharTableDialog: TCharTableDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{------------------------------------------------------------------- }
procedure TCharTableDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  DefFontName:=CharGrid.Font.Name;
  end;

procedure TCharTableDialog.CharGridDrawCell(Sender: TObject; ACol,
  ARow: Integer; Rect: TRect; State: TGridDrawState);
begin
  with CharGrid.Canvas do begin
    with Font do begin
      Name:=DefFontName; Size:=8; //Style:=[fsBold];
      end;
    if (ARow=0) and (ACol>0) then TextOut(Rect.Left+3,Rect.Top+3,HexByte(ACol-1));
    if (ACol=0) and (ARow>0) then TextOut(Rect.Left+3,Rect.Top+3,HexByte(ARow-1));
    with Font do begin
      Name:=GridFontName; Size:=8; Style:=[];
      end;
    if (ACol>0) and (ARow>0) then begin
      TextOut(Rect.Left+3,Rect.Top+3,chr(16*pred(ARow)+pred(ACol)));
      end;
    end;
  end;

procedure TCharTableDialog.CharGridDblClick(Sender: TObject);
begin
  ModalResult:=mrOK;
  end;

{------------------------------------------------------------------- }
function CharFromTable (AFontName : TFontName) : char;
begin
  if not assigned(CharTableDialog) then CharTableDialog:=TCharTableDialog.Create(Application);
  with CharTableDialog do begin
    GridFontName:=AFontName;
    if ShowModal=mrOK then with CharGrid do Result:=chr(16*pred(Row)+pred(Col))
    else Result:=#0;
    Release;
    end;
  CharTableDialog:=nil;
  end;

end.
