(* Delphi Dialog
   Show a black/white bitmap as 3D object using DirectX
   ====================================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jan. 2002
   *)

unit D3DShow;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Direct3D8, D3DX8, D3DX8Sub, D3DXTerrain, StdCtrls, ExtDlgs, Menus, IniFiles,
  JrUtils, ExtCtrls, ComCtrls, WinMath, Buttons;

const
  DefViewPoint : TPoint = (X : 500; Y : -750);
  DefDiff : TD3DColorValue = (r : 1.0; g : 1.0; b : 1.0; a : 1);
  DefAmbi : TD3DColorValue = (r : 0.4; g : 0.4; b : 0.4; a : 1);
  DefSpec : TD3DColorValue = (r : 0.8; g : 0.8; b : 0.8; a : 1);

  BmpExt = '.bmp';

  FullScrWidth = 640;     // Screen size for fullscreen mode
  FullScrHeight = 480;

type
  TShow3D = class(TForm)
    MainMenu: TMainMenu;
    BackItem: TMenuItem;
    RotYBar: TTrackBar;
    RotXBar: TTrackBar;
    RotZBar: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    SurfColorBar: TTrackBar;
    LightDirBar: TTrackBar;
    LightIntensBar: TTrackBar;
    View1: TMenuItem;
    View400Item: TMenuItem;
    View600Item: TMenuItem;
    View800Item: TMenuItem;
    N1: TMenuItem;
    ViewFullItem: TMenuItem;
    Label18: TLabel;
    Label19: TLabel;
    Label14: TLabel;
    SHeightBar: TTrackBar;
    Label13: TLabel;
    Label12: TLabel;
    Label16: TLabel;
    VDistBar: TTrackBar;
    VPitchBar: TTrackBar;
    Label15: TLabel;
    VYawBar: TTrackBar;
    Label24: TLabel;
    Label23: TLabel;
    Label22: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Save3DItem: TMenuItem;
    Texture1: TMenuItem;
    CreateTextureItem: TMenuItem;
    ShowTextureItem: TMenuItem;
    SavePictureDialog: TSavePictureDialog;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    SurfColorCB: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BackItemClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure RotBarChange(Sender: TObject);
    procedure ViewBarChange(Sender: TObject);
    procedure SurfColorBarChange(Sender: TObject);
    procedure AmbiBarChange(Sender: TObject);
    procedure View400ItemClick(Sender: TObject);
    procedure View600ItemClick(Sender: TObject);
    procedure View800ItemClick(Sender: TObject);
    procedure SHeightBarChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Save3DItemClick(Sender: TObject);
    procedure CreateTextureItemClick(Sender: TObject);
    procedure ShowTextureItemClick(Sender: TObject);
  private
    { Private-Deklarationen }
    D3D8             : IDIRECT3D8;
    D3DDev8          : IDirect3DDevice8;

    HwVertexProcess,
    FullScreen       : Boolean;
    BitsPerPixel     : integer;

    FIniFileName,
    FIniSection,
    TName,
    FName       : string;
    FormPos,
    ImgPos      : TPoint;
    ImgState    : integer;
    First,DXHal,
    DXReady     : boolean;
    Textur      : TBitmap;
    // Surface light
    Diff, Ambi,
    Spec        : TD3DColorValue;
    SpecPow     : integer;
    Light       : TD3DLight8;
    Terrain     : TD3DTerrain;
    TerrainPos  : TD3DVector;
    Boden       : IDirect3DTexture8;
    Bitmap      : TBitmap;

    function D3DInit (AHandle : THandle) : boolean;
    procedure SetViewMatrix (avp : TD3DXVector3);
    function ComputeViewPoint (Dist,pitch,yaw : integer) : TD3DXVector3;
    function GetColor (Hue,Bright,Refl : integer) : TD3DColorValue;
    procedure SetImageSize (State : integer);
  public
    procedure D3DInitScene;
    procedure D3DRender;
    procedure LoadFromIni (IniName, IniSection : string);
    procedure Execute (AName : string;
                       ABitmap : TBitmap;
                       ARect   : TRect);
  end;

var
  Show3D: TShow3D;

implementation

uses D3DImage, GTCDlg, gnugettext;

{$R *.DFM}

const
  iniLeft     = 'Left';
  iniTop      = 'Top';
  iniImgLeft  = 'ImgLeft';
  iniImgTop   = 'ImgTop';
  iniImgState = 'ImgState';
  iniViewDist = 'ViewDistance';
  iniViewPtch = 'ViewPitch';
  iniViewYaw  = 'ViewYaw';
  iniSurfHgt  = 'SurfaceHeight';
  iniSurfCol  = 'SurfaceColor';
  iniLightInt = 'LightIntensity';
  iniLightDir = 'LightDirection';
  iniTextur   = 'ShowTexture';

{ ------------------------------------------------------------------- }
procedure TShow3D.LoadFromIni (IniName, IniSection : string);
var
  n       : integer;
begin
  FIniFileName:=IniName;
  FIniSection:=IniSection;
  with TIniFile.Create(FIniFileName) do begin
    FormPos.X:=ReadInteger (IniSection,iniLeft,Left);
    FormPos.Y:=ReadInteger (IniSection,iniTop,Top);
    ImgPos.X:=ReadInteger (IniSection,iniImgLeft,50);
    ImgPos.Y:=ReadInteger (IniSection,iniImgTop,50);
    ImgState:=ReadInteger (IniSection,iniImgState,0);
    VDistBar.Position:=ReadInteger (IniSection,iniViewDist,-1000);
    VPitchBar.Position:=ReadInteger (IniSection,iniViewPtch,45);
    VYawBar.Position:=ReadInteger (IniSection,iniViewYaw,0);
    with SurfColorBar do begin
      n:=ReadInteger (IniSection,iniSurfCol,Position);
      SurfColorCB.Checked:=n>0;
      Position:=abs(n-1);
      end;
    with LightDirBar do Position:=ReadInteger (IniSection,iniLightDir,Position);
    with LightIntensBar do Position:=ReadInteger (IniSection,iniLightInt,Position);
    with SHeightBar do Position:=ReadInteger (IniSection,iniSurfHgt,Position);
    ShowTextureItem.Checked:=ReadBool (IniSection,iniTextur,false);
    Free;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FIniFileName:=''; FIniSection:=''; First:=true;
  Fname:=''; DXReady:=false; DXHal:=false; 
  Bitmap:=TBitmap.Create;
  VDistBar.Position:=1000;
  VPitchBar.Position:=45;
  VYawBar.Position:=0;
  Spec:=DefSpec;
  SpecPow:=0;
  D3D8:=nil;
  D3DDev8:=nil;
  FullScreen:=false;  // show in window
  BitsPerPixel:=16;   // 16 or 32 for fullscreen mode
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.FormDestroy(Sender: TObject);
var
  n       : integer;
begin
  DXReady := false;
  BitMap.Free;
  if assigned(D3DDev8) then D3DDev8:=nil;
  if assigned(D3D8) then D3D8:=nil;
  { Automatische Speicherung, wenn die notwendigen
    Angaben gemacht sind: }
  if (FIniFileName<>'') and (FIniSection<>'') then begin
    with TIniFile.Create(FIniFileName) do begin
      WriteInteger (FIniSection,iniLeft,Left);
      WriteInteger (FIniSection,iniTop,Top);
      WriteInteger (FIniSection,iniImgLeft,ImgPos.X);
      WriteInteger (FIniSection,iniImgTop,ImgPos.Y);
      WriteInteger (FIniSection,iniImgState,ImgState);
      WriteInteger (FIniSection,iniViewDist,VDistBar.Position);
      WriteInteger (FIniSection,iniViewPtch,VPitchBar.Position);
      WriteInteger (FIniSection,iniViewYaw,VYawBar.Position);
      n:=SurfColorBar.Position+1;
      if not SurfColorCB.Checked then n:=-n;
      WriteInteger (FIniSection,iniSurfCol,n);
      WriteInteger (FIniSection,iniLightDir,LightDirBar.Position);
      WriteInteger (FIniSection,iniLightInt,LightIntensBar.Position);
      WriteInteger (FIniSection,iniSurfHgt,SHeightBar.Position);
      WriteBool (FIniSection,iniTextur,ShowTextureItem.Checked);
      Free;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
function TShow3D.D3DInit (AHandle : THandle) : boolean;
var
  D3DDevCaps       : TD3DCaps8;
  D3DDispMode      : TD3DDisplayMode;
  D3DPrepar        : TD3DPresentParameters;
  dt               : TD3DDevType;
  vp               : integer;
begin
  Result:=false;
  D3D8:=Direct3DCreate8(D3D_SDK_VERSION);
  if D3D8=nil then exit;
  Fillchar(D3DPrePar,sizeof(TD3DPresentParameters),0);
  with D3DPrePar do begin
    SwapEffect:=D3DSWAPEFFECT_DISCARD;
    hDeviceWindow:=AHandle;
    EnableAutoDepthStencil := TRUE;
    AutoDepthStencilFormat := D3DFMT_D16;
    Windowed:=not FullScreen;
    if FullScreen then begin
      BackBufferWidth:=FullScrWidth;
      BackBufferHeight:=FullScrHeight;
      case BitsPerPixel of
        16 : BackBufferFormat:=D3DFMT_R5G6B5;
        32 : BackBufferFormat:=D3DFMT_A8R8G8B8;
        else begin
          D3D8:=nil; exit;
          end;
        end;
      BackBufferCount:=1;
      end
    else begin
      if failed(D3D8.GetAdapterDisplayMode(D3DADAPTER_DEFAULT,D3DDispMode)) then exit;
      BackBufferFormat:=D3DDispMode.Format;
      end;
    end;

  with D3D8 do begin
    // Hardware Renderer?
    if not failed(GetDeviceCaps(D3DADAPTER_DEFAULT,D3DDEVTYPE_HAL,D3dDevCaps)) then DXHal:=true
    else if not failed(GetDeviceCaps(D3DADAPTER_DEFAULT,D3DDEVTYPE_REF,D3dDevCaps)) then DXHal:=false
      else exit;
    // Hardware T&L?
    HwVertexProcess:=D3dDevCaps.DevCaps and D3DDEVCAPS_HWTRANSFORMANDLIGHT <> 0;
    if HwVertexProcess then vp:=D3DCREATE_HARDWARE_VERTEXPROCESSING
    else vp:=D3DCREATE_SOFTWARE_VERTEXPROCESSING;
    if DXHal then dt:=D3DDEVTYPE_HAL else dt:=D3DDEVTYPE_REF;

    if failed(CreateDevice(D3DADAPTER_DEFAULT,dt,AHandle,vp,
                           D3DPrePar,D3DDev8)) then begin
      D3D8:=nil; exit;
      end;
    end;
  Result:=true;
  end;

{ ------------------------------------------------------------------- }
// Set view point
procedure TShow3D.SetViewMatrix (avp : TD3DXVector3);
var
  lookat, from, worldup : TD3DXVector3;
  matView               : TD3DXMatrix;
begin
//  with avp do from := D3DXVector3(x,y,z);
  from:=avp;
  lookat := D3DXVector3(0,0,0);
  worldup := D3DXVector3(0,1,0);
  D3DXMatrixLookAtLH(matView, from, lookat, worldup);
  D3DDev8.SetTransform(D3DTS_VIEW, matView);
  end;

{ ------------------------------------------------------------------- }
// Init the 3D scene
procedure TShow3D.D3DInitScene;
var
  matProj  : TD3DXMatrix;
begin
  if DxReady then with d3ddev8 do begin
    SetRenderState(D3DRS_AMBIENT,d3dcolor_xrgb(128,128,128));
    SetRenderState(D3DRS_DITHERENABLE, 1);
    SetRenderState(D3DRS_SPECULARENABLE, 0);

    SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
    SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);

    SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);

    // set projection matrix
    D3DXMatrixPerspectiveFovLH(matProj, pi/4, pi/4, 1, 10000);
    SetTransform(D3DTS_PROJECTION, matProj);

    // set light (Directional Light)
    FillChar(Light, sizeof(Light), 0 );
    with Light do begin
      _Type := D3DLIGHT_DIRECTIONAL; // set to parallel light
      Ambient:=D3DXColor (1,1,1,0);  // set to white
      Diffuse:=D3DXColor (1,1,1,0);  // set to white
      Specular:=D3DXColor (1,1,1,0); // set to white
      // light direction in y-z-plane
      with Direction do begin
        z:=sin(-Pi180*LightDirBar.Position);
        y:=-cos(-Pi180*LightDirBar.Position);
        x:=0;
        end;
      Attenuation0:=1.0;     // Don't attenuate.
      Range:=100000.0;       // range of light
      end;
    // material properties
    if SurfColorCB.Checked then Diff:=GetColor(SurfColorBar.Position,100,100)
    else Diff:=DefDiff;
    Ambi:=ColorScale (DefAmbi,LightIntensBar.Position/100);

    //Generate terrain from bitmap
    with Terrain do begin
      AssignTerrain(Bitmap);
      //set terrain to center
      with BmpMap do TerrainPos:=D3DXVector3(-Width div 2,0,-Height div 2);
      GenerateTerrain (TerrainPos,SHeightBar.Position/100);
      end;

    // load texture
    D3DXCreateTextureFromFileA(D3DDev8,pchar(TName),boden);

    // set view matrix
    SetViewMatrix(ComputeViewPoint (VDistBar.Position,VPitchBar.Position,VYawBar.Position));
    end;
  end;

//##############################################################################
//  Render Procedure
//##############################################################################
procedure TShow3D.D3DRender;
var
  mtrl    : TD3DMaterial8;
  matWorld,
  mw1,mw2 : TD3DXMatrix;
begin
  if DXReady then begin
    if not DXHal then Screen.Cursor:=crHourglass;
    with D3DDev8 do begin
      Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(100, 100, 100), 1, 0);

      BeginScene;
      // set world matrix
      D3DXMatrixRotationYawPitchRoll(matWorld,RotYBar.Position*Pi180,RotXBar.Position*Pi180,RotZBar.Position*Pi180);
//    D3DXMatrixScaling(mw2,1,SHeightBar.Position/100,1);
//    D3DXMatrixMultiply(matWorld,mw2,matWorld);
      SetTransform(D3DTS_WORLD,matWorld);

      // set light
      SetLight(0,Light);
      LightEnable(0,true);

      // set material properties
      Fillchar(Mtrl,sizeof(TD3DMATERIAL8),0);
      with mtrl do begin
        Diffuse:=Diff;
        Ambient:=Ambi;
  //      Emissive:=Spec;
        Specular:=Spec;
        Power:=SpecPow;
        end;
      SetMaterial(mtrl);

      // set texture
      if ShowTextureItem.Checked then SetTexture(0,boden)
      else SetTexture(0,nil);
      SetVertexShader(D3DFVF_D3DVERTEX);

      // draw terrain
      if assigned(Terrain) then Terrain.Draw;

      EndScene;
      Present(nil, nil, 0, nil);
      end;

    if not DXHal then begin
      D3Image.ShowText(5,5,dgettext('dialogs','Using Software Renderer'));
      Screen.Cursor:=crDefault;
      end;
    end;
  BringToFront;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.BackItemClick(Sender: TObject);
begin
  D3Image.Close;
  Close;
end;

{ ------------------------------------------------------------------- }
procedure TShow3D.RotBarChange(Sender: TObject);
begin
  D3DRender;
  end;

{ ------------------------------------------------------------------- }
function TShow3D.ComputeViewPoint (Dist,pitch,yaw : integer) : TD3DXVector3;
begin
  with Result do begin
    X:=-round(Dist*cos(Pi180*pitch)*sin(Pi180*yaw));
    Z:=round(Dist*cos(Pi180*pitch)*cos(Pi180*yaw));
    Y:=-round(Dist*sin(Pi180*pitch));
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.ViewBarChange(Sender: TObject);
begin
  if DXReady then begin
    SetViewMatrix(ComputeViewPoint (VDistBar.Position,VPitchBar.Position,VYawBar.Position));
    D3DRender;
    end;
  end;

{ ------------------------------------------------------------------- }
function TShow3D.GetColor (Hue,Bright,Refl : integer) : TD3DColorValue;
var
  h  : integer;

  function ColorScaleInc (w,f : integer) : single;
  begin
    Result:=f*(1-sqr((w-60)/60))/100;
    end;

  function ColorScaleDec (w,f : integer) : single;
  begin
    Result:=f*(1-sqr(w/60))/100;
    end;

begin
  with Result do begin
    a:=Refl/100;
    h:=Hue mod 360;
    if h<=60 then begin
      r:=Bright/100; b:=0;
      g:=ColorScaleInc(h,Bright);
      end
    else if h<=120 then begin
      g:=Bright/100; b:=0;
      r:=ColorScaleDec(h-60,Bright);
      end
    else if h<=180 then begin
      g:=Bright/100; r:=0;
      b:=ColorScaleInc(h-120,Bright);
      end
    else if h<=240 then begin
      b:=Bright/100; r:=0;
      g:=ColorScaleDec(h-180,Bright);
      end
    else if h<=300 then begin
      b:=Bright/100; g:=0;
      r:=ColorScaleInc(h-240,Bright);
      end
    else begin
      r:=Bright/100; g:=0;
      b:=ColorScaleDec(h-300,Bright);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.SurfColorBarChange(Sender: TObject);
begin
  if SurfColorCB.Checked then Diff:=GetColor(SurfColorBar.Position,100,100)
  else Diff:=DefDiff;
  Ambi:=ColorScale (DefAmbi,LightIntensBar.Position/100);
  D3DRender;
  end;

procedure TShow3D.AmbiBarChange(Sender: TObject);
begin
  with Light.Direction do begin
    z:=sin(-Pi180*LightDirBar.Position);
    y:=-cos(-Pi180*LightDirBar.Position);
    end;
  D3DRender;
  end;

procedure TShow3D.SHeightBarChange(Sender: TObject);
begin
  if assigned(Terrain) then begin
    Terrain.GenerateTerrain (TerrainPos,SHeightBar.Position/100);
    D3DRender;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.SetImageSize (State : integer);
var
  size : integer;
begin
  ImgState:=State;
  case State of
  0 : begin
      View400Item.Checked:=true;
      size:=400;
      end;
  1 : begin
      View600Item.Checked:=true;
      size:=600;
      end;
  2 : begin
      View800Item.Checked:=true;
      size:=800;
      end;
    end;
  with D3Image do begin
    clientwidth:=size;
    clientheight:=size;
    end;
  D3DRender;
  end;

procedure TShow3D.View400ItemClick(Sender: TObject);
begin
  SetImageSize (0);
  end;

procedure TShow3D.View600ItemClick(Sender: TObject);
begin
  SetImageSize (1);
  end;

procedure TShow3D.View800ItemClick(Sender: TObject);
begin
  SetImageSize (2);
  end;

procedure TShow3D.Save3DItemClick(Sender: TObject);
begin
  Hide;
  with SavePictureDialog do begin
    InitialDir:=ExtractFilePath(FName);
    Filename:=AddNameSuffix(ExtractFileName(FName),'-3D','bmp');
    DefaultExt:='bmp';
    Filter:=dgettext('dialogs','Windows-Bitmap|*.bmp');
    Title:=dgettext('dialogs','Save 3D as Windows-Bitmap');
    if Execute then begin
      Application.ProcessMessages;  // close all dialog windows
      D3Image.SaveToFile(Filename);
      end;
    end;
  Show;
  end;

procedure TShow3D.CreateTextureItemClick(Sender: TObject);
begin
  if GTCDialog.Execute (FName,TName,Bitmap) then begin
    ShowTextureItem.Checked:=true;
    D3DXCreateTextureFromFileA(D3DDev8,pchar(TName),boden);
    D3DRender;
    end;
  end;

procedure TShow3D.ShowTextureItemClick(Sender: TObject);
begin
  with ShowTextureItem do Checked:=not Checked;
  D3DRender;
  end;

{ --------------------------------------------------------------------- }
procedure TShow3D.FormShow(Sender: TObject);
begin
  D3Image.ShowImage(FName,D3DRender);
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  DXReady:=false;
  end;

{ ------------------------------------------------------------------- }
procedure TShow3D.Execute (AName : string;
                           ABitmap : TBitmap;
                           ARect   : TRect);
begin
  Screen.Cursor:=crHourglass;
  with Bitmap do begin
    Assign(ABitmap);
    with ARect do begin
      Width:=Right-Left; Height:=Bottom-Top;
      end;
    Canvas.CopyRect(Rect(0,0,Width,Height),ABitmap.Canvas,ARect);
    end;
  FName:=AName;
  TName:=ExtractFilePath(FName)+AddNameSuffix(ExtractFileName(FName),'-Tx',BmpExt);
  if not assigned(D3Image) then D3Image:=TD3Image.Create(Application);
  with D3Image do begin
    SetImageSize (ImgState);
    with ImgPos do begin
      Left:=X; Top:=Y;
      end;
    end;
  if First then begin
    with FormPos do begin
      Left:=X; Top:=Y;
      end;
    First:=false;
    end;
  DxReady:=D3DInit(D3Image.Handle);
  if DxReady then begin
    if (not assigned(Terrain)) then Terrain:=TD3DTerrain.Create(D3DDev8);
    try
      D3DInitScene;
    except
      Screen.Cursor:=crDefault;
      Exit;
      end;
    Screen.Cursor:=crDefault;
    ShowModal;
    with D3Image do begin
      with ImgPos do begin
        X:=Left; Y:=Top;
        end;
      end;
    end
  else begin
    Screen.Cursor:=crDefault;
    ErrorDialog (Caption,dgettext('dialogs','Direct3D not supported!'));
    end;
  Terrain.Free; Terrain:=nil;
  D3Image.Release; D3Image:=nil;
  end;

end.
