(* Dialog fr doppelspaltige Wortlisten
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Okt. 2002
   *)

unit DListDlg;

interface

uses WinTypes, WinProcs, Classes, SysUtils, Graphics, Forms, Controls, Buttons,
  StdCtrls, Grids;

const
  WordLength = 79;
  MinLines = 12;
  CvLeftUp  = 1;
  CvLeft    = 2;
  CvRightUp = 4;
  CvRight   = 8;

type
  TWordString = string[WordLength];

  TDoubleString = class(TObject)
    String1,String2 : TWordString;
    Index           : integer;
    constructor Create (AString1,AString2 : string; AIndex : integer);
    end;

  TWortListDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    WortPaarListe: TStringGrid;
    Wort1Edit: TEdit;
    Wort2Edit: TEdit;
    Label1: TLabel;
    LabelLeft: TLabel;
    LabelRight: TLabel;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    DeleteBtn: TSpeedButton;
    InsertBtn: TSpeedButton;
    ReplBtn: TSpeedButton;
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure WortPaarListeSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure DeleteBtnClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure Wort2EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure KeyPress(Sender: TObject; var Key: Char);
    procedure ReplBtnClick(Sender: TObject);
    procedure Wort1EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure WortPaarListeClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FCvMode,
    LineCount : integer;
    procedure UpdateCells (ARow : integer);
  public
    { Public declarations }
    function Execute (Title,ALeft,ARight     : string;
                      LeftRatio,Index,CvMode : integer;
                      var AListe             : TStringList) : boolean;
  end;

var
  WortListDialog: TWortListDialog;

implementation

{$R *.DFM}

uses gnugettext;

{ ---------------------------------------------------------------- }
constructor TDoubleString.Create (AString1,AString2 : string;
                                  AIndex            : integer);
begin
  inherited Create;
  String1:=AString1; String2:=AString2; Index:=AIndex;
  end;

{ ---------------------------------------------------------------- }
procedure TWortListDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TWortListDialog.UpdateCells (ARow : integer);
begin
  with WortPaarListe do begin
    if (FCvMode and CvLeft<>0) then begin
      if (FCvMode and CvLeftUp)<>0 then cells[0,ARow]:=AnsiUpperCase(Wort1Edit.Text)
      else cells[0,ARow]:=AnsiLowerCase(Wort1Edit.Text);
      end
    else cells[0,ARow]:=Wort1Edit.Text;
    if (FCvMode and CvRight<>0) then begin
      if (FCvMode and CvRightUp)<>0 then cells[1,ARow]:=AnsiUpperCase(Wort2Edit.Text)
      else cells[1,ARow]:=AnsiLowerCase(Wort2Edit.Text);
      end
    else cells[1,ARow]:=Wort2Edit.Text;
    end;
  end;

{ ---------------------------------------------------------------- }
procedure TWortListDialog.UpBtnClick(Sender: TObject);
begin
  with WortPaarListe do if (Row>1) and (Row<=LineCount) then begin
    Cols[0].Exchange(Row,Row-1);
    Cols[1].Exchange(Row,Row-1);
    Row:=Row-1;
    Wort1Edit.Text:=cells[0,Row];
    Wort2Edit.Text:=cells[1,Row];
    end;
  Wort1Edit.SetFocus;
  end;

procedure TWortListDialog.DownBtnClick(Sender: TObject);
begin
  with WortPaarListe do if Row<LineCount then begin
    Cols[0].Exchange(Row,Row+1);
    Cols[1].Exchange(Row,Row+1);
    Row:=Row+1;
    Wort1Edit.Text:=cells[0,Row];
    Wort2Edit.Text:=cells[1,Row];
    end;
  Wort1Edit.SetFocus;
  end;

{ ---------------------------------------------------------------- }
procedure TWortListDialog.DeleteBtnClick(Sender: TObject);
var
  i : integer;
begin
  if LineCount>0 then begin
    dec(LineCount);
    with WortPaarListe do begin
      for i:=Row to Linecount do begin
        cells[0,i]:=cells[0,succ(i)];
        cells[1,i]:=cells[1,succ(i)];
        end;
      cells[0,succ(LineCount)]:='';
      cells[1,succ(LineCount)]:='';
      if (Row=succ(LineCount)) and (Row>1) then Row:=Row-1;
      if LineCount>MinLines then RowCount:=LineCount+1 else RowCount:=MinLines+1;
      end;
    end;
  Wort1Edit.SetFocus;
  end;

procedure TWortListDialog.InsertBtnClick(Sender: TObject);
var
  i : integer;
begin
  with WortPaarListe do begin
    if LineCount>=MinLines then RowCount:=LineCount+2 else RowCount:=MinLines+1;
    for i:=Linecount downto Row do begin
      cells[0,succ(i)]:=cells[0,i];
      cells[1,succ(i)]:=cells[1,i];
      end;
    UpdateCells(Row);
    end;
  inc(LineCount);
  Wort1Edit.SetFocus;
  end;

procedure TWortListDialog.ReplBtnClick(Sender: TObject);
begin
  with WortPaarListe do begin
    UpdateCells(Row);
   end;
  Wort1Edit.SetFocus;
  end;

{ ---------------------------------------------------------------- }
procedure TWortListDialog.WortPaarListeClick(Sender: TObject);
begin
  with WortPaarListe do begin
    Wort1Edit.Text:=cells[0,Row];
    Wort2Edit.Text:=cells[1,Row];
    end;
  end;

procedure TWortListDialog.WortPaarListeSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  CanSelect:=Row<=LineCount;
  end;

{ ---------------------------------------------------------------- }
procedure TWortListDialog.Wort2EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then begin
    InsertBtnClick(Sender);
    Wort1Edit.SetFocus;
    end;
  if (Key=VK_DOWN) then with WortPaarListe do if (Row<LineCount) then Row:=Row+1;
  if (Key=VK_Up) then with WortPaarListe do if (Row>1) then Row:=Row-1;
  end;

procedure TWortListDialog.KeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$0D then Key:=#0;
end;

procedure TWortListDialog.Wort1EditKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Key=VK_RETURN) then Wort2Edit.SetFocus;
  if (Key=VK_DOWN) then with WortPaarListe do if (Row<LineCount) then Row:=Row+1;
  if (Key=VK_Up) then with WortPaarListe do if (Row>1) then Row:=Row-1;
  end;

{ ---------------------------------------------------------------- }
(* LeftRatio : Breite des linken Feldes in %
   Index     : (1 oder 2) String fr Reihenfolge *)
function TWortListDialog.Execute (Title,ALeft,ARight     : string;
                                  LeftRatio,Index,CvMode : integer;
                                  var AListe             : TStringList) : boolean;
var
  so,ok : boolean;
  dp    : TDuplicates;
  i     : integer;
begin
  Caption:=Title;
  LabelLeft.Caption:=ALeft+':'; LabelRight.Caption:=ARight+':';
  FCvMode:=CvMode;
  // Spaltenbreiten einstellen (Wert in %)
  if LeftRatio>0 then with WortPaarListe do begin
    ColWidths[0]:=round(LeftRatio*(Width-3)/100);
    ColWidths[1]:=Width-3-ColWidths[0];
    Wort1Edit.Width:=ColWidths[0]-5;
    i:=Left+ColWidths[0]+5;
    with Wort2Edit do begin
      Left:=i; Width:=ColWidths[1]-5;
      end;
    LabelRight.Left:=i;
    end;
  if Index<1 then Index:=1;
  if Index>2 then Index:=2;
  with WortPaarListe do begin
    with AListe do begin
      LineCount:=Count;
      if LineCount>MinLines then RowCount:=LineCount+1 else RowCount:=MinLines+1;
      UpBtn.Visible:=not Sorted;
      DownBtn.Visible:=not Sorted;
      for i:=1 to RowCount-1 do if i<=Count then
          with Objects[i-1] as TDoubleString do begin
        cells[0,i]:=String1; cells[1,i]:=String2;
        end
      else begin
        cells[0,i]:=''; cells[1,i]:='';
        end;
      end;
    cells[0,0]:=ALeft;
    cells[1,0]:=ARight;
    Row:=1;
    Wort1Edit.Text:=cells[0,Row]; Wort2Edit.Text:=cells[1,Row];
    end;
  ok:=ShowModal=mrOK;
  if ok then begin
    with AListe do begin
      so:=Sorted; dp:=Duplicates; Free;
      end;
    AListe:=TStringList.Create;
    with AListe do begin
      Sorted:=so; Duplicates:=dp;
      with WortPaarListe do for i:=1 to LineCount do
        AddObject(cells[Index-1,i],TDoubleString.Create(cells[0,i],cells[1,i],0));
      end;
    end;
  ActiveControl:=Wort1Edit;  
  Result:=ok;
  end;

end.
