(* Eingabedialog fr eine verkettete Stringliste
   Hauptliste :  Gruppen
   Unterlisten:  Items pro Gruppeneintrag
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
       
   Vers. 1 - Okt. 2003 
   *)

unit DStringListDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Grids, Dialogs;

const
  DefRowCount = 21;

type
  TDescString = string[39];

  TDStringListDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    sgrItems: TStringGrid;
    spdLoad: TSpeedButton;
    spdEdit: TSpeedButton;
    spdNew: TSpeedButton;
    spdRem: TSpeedButton;
    lbxGruppen: TListBox;
    OpenDialog: TOpenDialog;
    labGruppen: TLabel;
    labItems: TLabel;
    procedure spdLoadClick(Sender: TObject);
    procedure lbxGruppenClick(Sender: TObject);
    procedure spdNewClick(Sender: TObject);
    procedure spdEditClick(Sender: TObject);
    procedure spdRemClick(Sender: TObject);
    procedure sgrItemsSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure sgrItemsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    OldNdx   : integer;
    Hd1,Hd2  : string;
    ItemCols : integer;
    ShortCuts : boolean;
    procedure ShowItems;
  public
    { Public declarations }
    function LoadFile (FName : string; var FTL : TStringList) : boolean;
    procedure SaveFile (FName : string; FTL : TStringList);
    function Execute (Titel,GrpTitel,ItemTitel,ItemHeader1,ItemHeader2 : string;
                      AShortCuts : boolean; var ATL : TStringList) : boolean;
  end;

var
  DStringListDialog: TDStringListDialog;

implementation

{$R *.DFM}

uses JrUtils, WinShell, ShlObj, StrUtils, TextDlg, gnugettext;

{ ------------------------------------------------------------------- }
procedure TDStringListDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

// Laden der Teilelisten
function TDStringListDialog.LoadFile (FName : string;
                                      var FTL : TStringList) : boolean;
var
  fs    : TFileStream;
  i,j,n,
  m,k   : integer;
  s,t   : TDescString;
  sl    : TStringList;
begin
  OldNdx:=-1;
  if FileExists(FName) then begin
    fs:=TFileStream.Create(FName,fmOpenRead);
    try
      FreeListObjects(FTL);
      with FTL do begin
        Clear; Sorted:=true;
        fs.ReadBuffer(n,sizeof(integer));
        if n>0 then for i:=1 to n do begin
          fs.ReadBuffer(s,sizeof(TDescString));
          fs.ReadBuffer(m,sizeof(integer));
          sl:=TStringList.Create;
          sl.Sorted:=true;
          if m>0 then with sl do for j:=1 to m do begin
            fs.ReadBuffer(t,sizeof(TDescString));
            fs.ReadBuffer(k,sizeof(integer));
            AddObject(t,TObject(k));
            end;
          AddObject(s,sl);
          end;
        end;
      Result:=true;
    finally
      fs.Free;
      end;
    end
  else Result:=false;
  end;

procedure TDStringListDialog.SaveFile (FName : string;
                                       FTL : TStringList);
var
  fs    : TFileStream;
  i,j,n : integer;
  s     : TDescString;
begin
  fs:=TFileStream.Create(FName,fmCreate);
  try
    with FTL do begin
      n:=Count;
      fs.WriteBuffer(n,sizeof(integer));
      if Count>0 then for i:=0 to Count-1 do begin
        FillChar(s,sizeof(TDescString),0); s:=Strings[i];
        fs.WriteBuffer(s,sizeof(TDescString));
        if assigned(Objects[i]) then begin
          with (Objects[i] as TStringList) do begin
            n:=Count;
            fs.WriteBuffer(n,sizeof(integer));
            if n>0 then for j:=0 to Count-1 do begin
              FillChar(s,sizeof(TDescString),0); s:=Strings[j];
              fs.WriteBuffer(s,sizeof(TDescString)); // Teilebez.
              n:=integer(Objects[j]);
              fs.WriteBuffer(n,sizeof(integer));     // Preis in Cent
              end;
            end;
          end
        else begin
          n:=0;
          fs.WriteBuffer(n,sizeof(integer));
          end;
        end;
      end;
  finally
    fs.Free;
    end;
  end;

// Daten importieren  (nicht sichtbar)
procedure TDStringListDialog.spdLoadClick(Sender: TObject);
var
  ft  : TextFile;
  s,t,u : string;
  k1,k2,k3,
  k4,k5  : integer;
  sl     : TStringList;
begin
  with Opendialog do begin
    Filter:=dgettext('dialogs','all|*.*')
    InitialDir:=GetDesktopFolder(CSIDL_PERSONAL);
    if Execute then begin
      s:=''; lbxGruppen.Clear; OldNdx:=-1;
      AssignFile (ft,FileName); reset(ft);
      while not Eof(ft) do begin
        readln (ft,t);
        s:=s+t;
        end;
      CloseFile(ft);
      k3:=1;
      repeat
        k1:=PosEx ('<a name="',s,k3);
        if k1>0 then begin
          k2:=PosEx('<b>',s,k1);
          if k2>0 then begin
            k3:=PosEx('<',s,k2+3);
            if s[k3-1]=':' then dec(k3);
            t:=Trim(copy(s,k2+3,k3-k2-3));
            if length(t)=0 then t:=dgettext('dialogs','unknown');
            k2:=PosEx('<br>',s,k3)+4;
            sl:=TStringList.Create;
            repeat
              if s[k2]='-' then inc(k2);
              k4:=PosEx('<',s,k2);
              k5:=PosEx(';',s,k2);
              if (k5>0) and (k5<k4) then begin   // Komma
                if k5>0 then begin
                  u:=trim(copy(s,k2,k5-k2));
                  if length(u)>0 then sl.Add(u);
                  k2:=k5+1;
                  end;
                end
              else begin
                if k4>0 then begin   // Tag
                  u:=trim(copy(s,k2,k4-k2));
                  if length(u)>0 then sl.Add(u);
                  if copy (s,k4,4)='<hr>' then k4:=0  // Ende
                  else k2:=PosEx('>',s,k4)+1;
                  end;
                end;
              until k4=0;
            k3:=k2;
            lbxGruppen.Items.AddObject(t,sl);
            end;
          end;
        until k1=0;
      end;
    end;
  ShowItems;
  end;

{ ------------------------------------------------------------------- }
procedure TDStringListDialog.ShowItems;
var
  i,n : integer;
begin
  with sgrItems do begin
    ColCount:=ItemCols;
    Cols[0].Clear;
    if ItemCols=2 then Cols[1].Clear;
    RowCount:=DefRowCount;
    Cells[0,0]:=Hd1;
    if ItemCols=2 then begin
      Cells[1,0]:=Hd2;
      ColWidths[1]:=65;
      ColWidths[0]:=Width-87;
      end
    else ColWidths[0]:=Width-21;
    Row:=1; Col:=0;
    end;
  with lbxGruppen do if (ItemIndex>=0) and assigned(Items.Objects[ItemIndex]) then
       with (Items.Objects[ItemIndex] as TStringList) do begin
    with sgrItems do if Count>RowCount-1 then RowCount:=Count+1;
    for i:=1 to Count do begin
      n:=integer(Objects[i-1]);
      with sgrItems do if length(Strings[i-1])>0 then begin
        cells[0,i]:=Strings[i-1];
        if ItemCols=2 then begin
          cells[1,i]:=FloatToStrF(n/100,ffFixed,8,2);
          objects[1,i]:=TObject(n);
          end;
        end;
      end;
    end;
  OldNdx:=lbxGruppen.ItemIndex;
  end;

procedure TDStringListDialog.lbxGruppenClick(Sender: TObject);
var
  i   : integer;
  x   : double;
begin
  // Teileliste updaten
  if (OldNdx>=0) and (OldNdx<lbxGruppen.Items.Count) then with sgrItems do begin
    if ItemCols=2 then for i:=1 to RowCount do if (length(cells[0,i])>0) then begin
      try
        if length(cells[1,i])>0 then x:=StrToFloat(cells[1,i]) else x:=0;
        objects[1,i]:=TObject(round(x*100));
      except
        on EConvertError do begin
          ErrorDialog ('',Format(dgettext('dialogs','Error in line %s'),[IntToStr(i)]));
          Row:=i; lbxGruppen.ItemIndex:=OldNdx;
          sgrItems.SetFocus;
          exit;
          end;
        end;
      end;
    with lbxGruppen.Items do begin
      if not assigned(Objects[OldNdx]) then Objects[OldNdx]:=TStringList.Create;
      with (Objects[OldNdx] as TStringList) do begin
        Clear;
        for i:=1 to RowCount do if (length(cells[0,i])>0) then begin
          if ItemCols=2 then AddObject (cells[0,i],sgrItems.objects[1,i])
          else Add(cells[0,i]);
          end;
        end;
      end;
    end;
  ShowItems;
  end;

procedure TDStringListDialog.sgrItemsSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
  with sgrItems do if Row=RowCount-1 then RowCount:=RowCount+1;
  end;

procedure TDStringListDialog.spdNewClick(Sender: TObject);
var
  s,t : string;
  ok  : boolean;
begin
  s:=''; t:='';
  if ShortCuts then ok:=DTextDialog(Hd1,dgettext('dialogs','Name:'),dgettext('dialogs','Shortcut:'),-1,-1,s,t)
  else ok:=InputQuery (Hd1,rsName,s);
  if ok then begin
    with lbxGruppen do begin
      lbxGruppenClick(Sender);
      if ShortCuts then s:=t+'|'+s;
      ItemIndex:=Items.AddObject(s,TStringList.Create);
      ShowItems;
      end;
    end;
  end;

procedure TDStringListDialog.spdEditClick(Sender: TObject);
var
  s,t : string;
  ok  : boolean;
begin
  with lbxGruppen do s:=Items[ItemIndex];
  if ShortCuts then begin
    t:=ReadNxtStr(s,'|');
    ok:=DTextDialog(Hd1,rsName,rsShortcut,-1,-1,s,t)
    end
  else ok:=InputQuery (Hd1,rsName,s);
  if ok then begin
    with lbxGruppen do begin
      if ShortCuts then Items[ItemIndex]:=t+'|'+s
      else Items[ItemIndex]:=s;
      end;
    end;
  end;

procedure TDStringListDialog.spdRemClick(Sender: TObject);
var
  n : integer;
begin
  with lbxGruppen do if ConfirmDialog ('',Format(dgettext('dialogs','Remove group: "%s"?'),[Items[ItemIndex]])) then begin
    with Items do begin
      n:=ItemIndex; Delete(n);
      OldNdx:=-1;
      if (Count>0) then begin
        if (n<Count)then ItemIndex:=n else ItemIndex:=Count-1;
        end;
      end;
    lbxGruppenClick(Sender);
    OldNdx:=ItemIndex;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Benutzername und Passwort, Ergebnis: "true" bei "ok" *)
function TDStringListDialog.Execute (Titel,GrpTitel,ItemTitel,ItemHeader1,ItemHeader2 : string;
                                     AShortCuts : boolean; var ATL : TStringList) : boolean;
begin
  Caption:=Titel;
  labGruppen.Caption:=GrpTitel;
  labItems.Caption:=ItemTitel;
  Hd1:=ItemHeader1;
  Hd2:=ItemHeader2;
  ShortCuts:=AShortCuts;
  if length(Hd2)>0 then ItemCols:=2 else ItemCols:=1;
  OldNdx:=-1;
  with lbxGruppen do begin
    Items.Assign(ATL);
    ItemIndex:=0;
    end;
  ShowItems;
  if ShowModal=mrOK then begin
    lbxGruppenClick(self);
//    FreeListObjects(ATL);
    ATL.Assign(lbxGruppen.Items);
    Result:=true;
    end
  else Result:=false;
  end;

procedure TDStringListDialog.sgrItemsKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  with sgrItems do if (Key=VK_Down) and (Row>RowCount-2) then RowCount:=RowCount+1;
  end;

end.
