(* Delphi Dialog
   Eingabe und Auswahl von Feiertagen fr einen Kalender
   =====================================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jan. 2005
   *)
   
unit EditHolidayDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, NumberEd, ComCtrls, DateUtilEx;

type
  THolidayObject = class (TObject)
    FData   : THoliday;
    constructor Create (AData : THoliday);
    property Data : THoliday read FData write FData;
    end;

  TEditHolidayDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    cbxHolidays: TComboBox;
    rgpType: TRadioGroup;
    pctType: TPageControl;
    btnNew: TBitBtn;
    btnDelete: TBitBtn;
    tshDate: TTabSheet;
    tshOffset: TTabSheet;
    Label1: TLabel;
    Label2: TLabel;
    edtDay: TRangeEdit;
    edtMonth: TRangeEdit;
    edtOffs: TRangeEdit;
    edtMult2: TRangeEdit;
    Label3: TLabel;
    Label4: TLabel;
    edtMult1: TRangeEdit;
    procedure rgpTypeClick(Sender: TObject);
    procedure cbxHolidaysChange(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    LastIndex : integer;
  public
    { Public declarations }
    function Execute (var AHolidayCalendar : THolidayCalendar; AName : string) : boolean;
  end;

var
  EditHolidayDialog: TEditHolidayDialog;

implementation

{$R *.DFM}

uses Dialogs, JrUtils, gnugettext;

{------------------------------------------------------------------- }
constructor THolidayObject.Create (AData : THoliday);
begin
  inherited Create;
  FData:=AData;
  end;

{------------------------------------------------------------------- }
procedure TEditHolidayDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TEditHolidayDialog.cbxHolidaysChange(Sender : TObject);
var
  h : THoliday;
begin
  if (cbxHolidays.Items.Count>0) and (LastIndex>=0) then begin
   h:=(cbxHolidays.Items.Objects[LastIndex] as THolidayObject).Data;
   with h do begin
     HType:=THolidayType(rgpType.ItemIndex);
     if HType=htFix then begin
       Day:=edtMonth.Value shl 8 or edtDay.Value;
       Mult:=edtMult1.Value;
       end
     else begin
       Offset:=edtOffs.Value;
       Mult:=edtMult2.Value;
       end;
     end;
   (cbxHolidays.Items.Objects[LastIndex] as THolidayObject).Data:=h;
   end;
  with cbxHolidays do if ItemIndex>=0 then begin
    LastIndex:=ItemIndex;
    h:=(Items.Objects[ItemIndex] as THolidayObject).Data;
    rgpType.ItemIndex:=integer(h.HType);
    rgpTypeClick(Sender);
    end;
  end;

procedure TEditHolidayDialog.rgpTypeClick(Sender: TObject);
var
  h : THoliday;
begin
  with cbxHolidays do h:=(Items.Objects[ItemIndex] as THolidayObject).Data;
  if rgpType.ItemIndex=0 then begin
    pctType.ActivePageIndex:=0;
    h.HType:=THolidayType(rgpType.ItemIndex);
    with h do begin
      edtDay.Value:=Day and $FF;
      edtMonth.Value:=(Day shr 8)and $FF;
      edtMult1.Value:=Mult;
      end;
    if Active then edtDay.SetFocus;
    end
  else begin
    pctType.ActivePageIndex:=1;
    with h do begin
      edtOffs.Value:=Offset;
      edtMult2.Value:=Mult;
      end;
    if Active then edtOffs.SetFocus;
    end;
  end;

{------------------------------------------------------------------- }
procedure TEditHolidayDialog.btnNewClick(Sender: TObject);
var
  s : string;
  h : THoliday;
begin
  cbxHolidaysChange(Sender);
  if InputQuery (dgettext('dialogs', 'New holiday'),:('Name:'),s) then begin
    with h do begin
      Name:=s; Active:=false; HType:=htFix; Day:=$101; Mult:=0; Offset:=0;
      end;
    with cbxHolidays do ItemIndex:=Items.AddObject(s,THolidayObject.Create(h));
    cbxHolidaysChange(Sender);
    end;
  end;

procedure TEditHolidayDialog.btnDeleteClick(Sender: TObject);
var
  n : integer;
begin
  with cbxHolidays do if (Items.Count>0) and
    ConfirmDialog('',Format(dgettext('dialogs','Remove "%s"'+sLineBreak+
                     'from list?'),[Items[ItemIndex]])) then begin
    n:=ItemIndex;
    Items.Delete(ItemIndex);
    if Items.Count<=0 then begin
      ItemIndex:=-1; Text:='';
      end
    else if n>=Items.Count then ItemIndex:=Items.Count-1
    else ItemIndex:=n;
    end;
  end;

{------------------------------------------------------------------- }
(* Dialog an Position anzeigen *)
function TEditHolidayDialog.Execute (var AHolidayCalendar : THolidayCalendar;
                                     AName : string) : boolean;
var
  i,n   : integer;
  h     : THolidayObject;
begin
  cbxHolidays.Clear;
  with AHolidayCalendar do for i:=0 to HolidayCount-1 do begin
    h:=THolidayObject.Create (Holiday[i]);
    cbxHolidays.Items.AddObject(Holiday[i].Name,h);
    end;
  with cbxHolidays do begin
    n:=Items.IndexOf(AName);
    if n>=0 then ItemIndex:=n
    else if Items.Count>0 then ItemIndex:=0 else ItemIndex:=-1;
    end;
  LastIndex:=-1;
  cbxHolidaysChange(self);
  if ShowModal=mrOK then with cbxHolidays do begin
    cbxHolidays.ItemIndex:=-1;
    cbxHolidaysChange(self);
    with AHolidayCalendar do begin
      HolidayCount:=Items.Count;
      for i:=0 to Items.Count-1 do
        Holiday[i]:=(Items.Objects[i] as THolidayObject).Data;
      Update;
      end;
    Result:=true;
    end
  else Result:=false;
  end;

end.
