(* Einstelldialog fr Etikettenlayout
   ==================================
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1
   *)

unit EtikettDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Spin, JrUtils;

const
  Rand = 5;

type
  TPageLayout = record
    PrSize,                     // Papiergre in mm einschl. Rand
    MinMarg,                    // Mindestrand in mm
    UsableSize,                 // nutzbare Gre in mm
    Margin,                     // Rand links/oben zu den Etiketten in mm
    Size,                       // Gre der Etiketten in mm
    Distance,                   // Abstand zwischen den Etiketten in mm
    Count            : TPoint;  // Anzahl der Etiketten hor. und dvert.
    ShowFrame        : boolean; // mit/ohne Linien
    end;

  TEtikettenDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Panel1: TPanel;
    Image1: TImage;
    GroupBox1: TGroupBox;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    GroupBox2: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label6: TLabel;
    spdHR: TSpinEdit;
    Label7: TLabel;
    spdHB: TSpinEdit;
    Label8: TLabel;
    Label9: TLabel;
    spdHA: TSpinEdit;
    Label10: TLabel;
    spdVR: TSpinEdit;
    spdVH: TSpinEdit;
    spdVA: TSpinEdit;
    Label11: TLabel;
    Label12: TLabel;
    Label13: TLabel;
    spdHC: TSpinEdit;
    spdVC: TSpinEdit;
    Label14: TLabel;
    Label15: TLabel;
    Panel2: TPanel;
    PaintBox: TPaintBox;
    cbFrame: TCheckBox;
    procedure PaintBoxPaint(Sender: TObject);
    procedure spdChange(Sender: TObject);
    procedure spdHAChange(Sender: TObject);
    procedure spdVAChange(Sender: TObject);
    procedure spdVHChange(Sender: TObject);
    procedure spdHBChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    PWidth : integer;
    FPage  : TPageLayout;
    function SetScreenScale : integer;
  public
    { Public declarations }
    function Execute (Title      : string;
                      var Page   : TPageLayout) : boolean;
  end;

var
  EtikettenDialog : TEtikettenDialog;

implementation

{$R *.DFM}

uses gnugettext;

{ ------------------------------------------------------------------- }
procedure TEtikettenDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

function TEtikettenDialog.SetScreenScale : integer;
begin
  with PaintBox,Canvas do begin
    Result:=SetMapMode (Handle,MM_ISOTROPIC);
    SetWindowOrgEx (Handle,0,0,nil);
    SetWindowExtEx (Handle,PWidth,PWidth,nil);
    SetViewPortOrgEx (Handle,0,0,nil);
    SetViewportExtEX (handle,Width,Height,nil);
    end;
  end;

procedure TEtikettenDialog.PaintBoxPaint(Sender: TObject);
var
  i,j,
  x0,y0,
  x1,y1 : integer;
begin
  SetScreenScale;
  with PaintBox.Canvas do begin
    Brush.Color:=clWhite;
    FillRect(Rect(0,0,PWidth,PWidth));
    with Pen do begin
      Color:=clBlack; Width:=1;
      end;
    with FPage.PrSize do begin
      x0:=(PWidth-x) div 2;
      y0:=(PWidth-y) div 2;
      Rectangle(x0,y0,x0+x,y0+y);
      for i:=0 to spdHC.Value-1 do
        for j:=0 to spdVC.Value-1 do begin
          x1:=x0+spdHR.Value+i*spdHA.Value;
          y1:=y0+spdVR.Value+j*spdVA.Value;
          Rectangle(x1,y1,x1+spdHB.Value,y1+spdVH.Value);
          end;
      end;
    end;
  end;

procedure TEtikettenDialog.spdChange(Sender: TObject);
begin
  PaintBox.Invalidate;
  end;

procedure TEtikettenDialog.spdHAChange(Sender: TObject);
begin
  if spdHB.Value>spdHA.Value then spdHB.Value:=spdHA.Value;
  PaintBox.Invalidate;
  end;

procedure TEtikettenDialog.spdHBChange(Sender: TObject);
begin
  if spdHB.Value>spdHA.Value then spdHA.Value:=spdHB.Value;
  PaintBox.Invalidate;
  end;

procedure TEtikettenDialog.spdVAChange(Sender: TObject);
begin
  if spdVH.Value>spdVA.Value then spdVH.Value:=spdVA.Value;
  PaintBox.Invalidate;
  end;

procedure TEtikettenDialog.spdVHChange(Sender: TObject);
begin
  if spdVH.Value>spdVA.Value then spdVA.Value:=spdVH.Value;
  PaintBox.Invalidate;
  end;

{ ------------------------------------------------------------------- }
function TEtikettenDialog.Execute (Title      : string;
                                   var Page   : TPageLayout) : boolean;
begin
  if length(Title)>0 then Caption:=Title;
  FPage:=Page;
  with Page do begin
    with PrSize do begin
      if x>y then PWidth:=x else PWidth:=y;
      end;
    PWidth:=PWidth+2*Rand;
    spdHR.Value:=Margin.x;
    spdVR.Value:=Margin.y;
    spdHB.Value:=Size.x;
    spdVH.Value:=Size.y;
    spdHA.Value:=Distance.x;
    spdVA.Value:=Distance.y;
    spdHC.Value:=Count.x;
    spdVC.Value:=Count.y;
    cbFrame.Checked:=ShowFrame;
    end;
  if ShowModal=mrOK then begin
    with Page do begin
      Margin.x:=spdHR.Value;
      Margin.y:=spdVR.Value;
      Size.x:=spdHB.Value;
      Size.y:=spdVH.Value;
      Distance.x:=spdHA.Value;
      Distance.y:=spdVA.Value;
      Count.x:=spdHC.Value;
      Count.y:=spdVC.Value;
      ShowFrame:=cbFrame.Checked;
      end;
    Result:=true;
    end
  else Result:=false;
  end;

end.
