(* Delphi Dialog
   Stringliste aus Dateinamen erstellen und sortieren
   ==================================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Okt. 2005
   *)

unit FileListDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Dialogs;

type
  TFileListDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    lbxFiles: TListBox;
    lblPath: TLabel;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    DeleteBtn: TSpeedButton;
    InsertBtn: TSpeedButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure InsertBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    function Execute (Title,FileFilter,Path,FName,ListFilter,DefExt : string) : string;
  end;

var
  FileListDialog: TFileListDialog;

implementation

{$R *.dfm}

uses gnugettext;

{------------------------------------------------------------------- }
procedure TFileListDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TFileListDialog.InsertBtnClick(Sender: TObject);
var
  i : integer;
begin
  with OpenDialog do if Execute then with Files do for i:=0 to Count-1 do
      lbxFiles.Items.Add(ExtractRelativePath(lblPath.Caption,Strings[i]));
  end;

procedure TFileListDialog.DeleteBtnClick(Sender: TObject);
var
  i : integer;
begin
  with lbxFiles do if (Items.Count>0) and (ItemIndex>0) then begin
    i:=ItemIndex;
    Items.Delete(Itemindex);
    if i<Items.Count then Itemindex:=i else Itemindex:=Items.Count-1;
    end;
  end;

procedure TFileListDialog.UpBtnClick(Sender: TObject);
var
  i : integer;
begin
  with lbxFiles do if (Items.Count>0) and (ItemIndex>0) then begin
    i:=ItemIndex-1;
    Items.Exchange (i,ItemIndex);
    SetFocus;
    ItemIndex:=i;
    end;
  end;

procedure TFileListDialog.DownBtnClick(Sender: TObject);
var
  i : integer;
begin
  with lbxFiles do if (Items.Count>0) and (ItemIndex<Items.Count-1) then begin
    i:=ItemIndex+1;
    Items.Exchange (i,ItemIndex);
    SetFocus;
    ItemIndex:=i;
    end;
  end;

{------------------------------------------------------------------- }
function TFileListDialog.Execute (Title,FileFilter,Path,FName,ListFilter,DefExt : string) : string;
var
  bf : TextFile;
  s  : string;
  i  : integer;
begin
  Caption:=Title;
  lblPath.Caption:=IncludeTrailingPathDelimiter(Path);
  lbxFiles.Clear;
  with OpenDialog do begin
    InitialDir:=Path;
    Filter:=FileFilter;
    end;
  with SaveDialog do begin
    InitialDir:=Path;
    Filter:=ListFilter;
    DefaultExt:=DefExt;
    end;
  if length(FName)>0 then begin
    AssignFile(bf,FName); reset(bf);
    while not Eof(bf) do begin
      readln (bf,s);
      lbxFiles.Items.Add(s);
      end;
    CloseFile(bf);
    lbxFiles.Itemindex:=0;
    end;
  if (ShowModal=mrOK) and (lbxFiles.Items.Count>0) then begin
    if length(FName)=0 then with SaveDialog do begin
      if Execute then FName:=Filename;
      end;
    if length(FName)>0 then begin
      AssignFile(bf,FName); rewrite(bf);
      with lbxFiles.Items do for i:=0 to Count-1 do writeln(bf,Strings[i]);
      CloseFile(bf);
      Result:=FName;
      end
    else Result:='';
    end
  else Result:='';
  end;

end.
