(* Delphi Dialog
   Auswahl von Filterbedingungen (siehe Unit TextFilter)
   =====================================================
   
   untersttzte Filtertypen:
   - Standard-Delphi (function MatchesMask)
   - Regulre Ausdrcke
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Dez. 2004
   letzte nderung: Sep. 2008
   *)

unit FilterDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, TextFilter, HListBox;

const
  MExp1 = '*';
  MExp2 = '[ab]*';
  RExp1 = '\D';
  RExp2 = 'A+[bc].';

type
  TFilterDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    lbMask: TLabel;
    IncMaskBtn: TRadioButton;
    ExcMaskBtn: TRadioButton;
    NoMaskBtn: TRadioButton;
    RegExpCB: TCheckBox;
    cbxFilter: THistoryCombo;
    btnEditMask: TBitBtn;
    procedure NoMaskBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnEditMaskClick(Sender: TObject);
  private
    { Private declarations }
    FFilter : TTextFilter;
  public
    { Public declarations }
    procedure LoadFromIni(IniName,IniSection : string);
    function Execute (Titel : string; ShowListBtn : boolean;
                      var AFilter : TTextFilter) : boolean;
  end;

var
  FilterDialog: TFilterDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses JrUtils, GnuGetText, SelectFromListDlg;

{ ---------------------------------------------------------------- }
procedure TFilterDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  cbxFilter.Style:=csSimple;
  end;

procedure TFilterDialog.LoadFromIni(IniName,IniSection : string);
begin
  cbxFilter.LoadFromIni(IniName,IniSection);
  end;

procedure TFilterDialog.FormShow(Sender: TObject);
begin
  with cbxFilter do begin
    if Items.Count=0 then Style:=csSimple else Style:=csDropDown;
    Enabled:=not NoMaskBtn.Checked;
    end;
  NoMaskBtnClick(self);
  end;

{ ---------------------------------------------------------------- }
procedure TFilterDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) then with FFilter do begin
    with cbxFilter do if Text='' then begin
      Regular:=false;
      Text:=MatchAll;
      end
    else begin
      Regular:=RegExpCB.Checked;
      end;
    if IncMaskBtn.Checked then Mode:=fmInclude
    else if ExcMaskBtn.Checked then Mode:=fmExclude
    else Mode:=fmOff;
    if not NoMaskBtn.Checked then begin
      try
        Filter:=cbxFilter.Text;
        with cbxFilter do AddItem(Text);
      except
        CanClose:=false;
        if Regular then ErrorDialog(Caption,
          Format(dgettext('dialogs','Invalid filter mask!'+sLineBreak+
                          'Should look like %s or %s'),
          [RExp1,RExp2]),CenterPos)
        else ErrorDialog(Caption,
          Format(dgettext('dialogs','Invalid filter mask!'+sLineBreak+
                          'Should look like %s or %s'),
          [MExp1,MExp2]),CenterPos);
        end;
      end;
    end;
  end;

procedure TFilterDialog.NoMaskBtnClick(Sender: TObject);
begin
  cbxFilter.Enabled:=not NoMaskBtn.Checked;
  btnEditMask.Enabled:=cbxFilter.Enabled;
  RegExpCB.Enabled:=not NoMaskBtn.Checked;
  if not NoMaskBtn.Checked then cbxFilter.SetFocus;
  end;

function TFilterDialog.Execute (Titel : string; ShowListBtn : boolean;
                                var AFilter: TTextFilter) : boolean;
begin
  Caption:=Titel;
  btnEditMask.Visible:=ShowListBtn;
  FFilter:=AFilter;
  with FFilter do begin
    cbxFilter.Text:=Filter;
    RegExpCB.Checked:=Regular;
    case Mode of
    fmInclude : IncMaskBtn.Checked:=true;
    fmExclude : ExcMaskBtn.Checked:=true;
      else NoMaskBtn.Checked:=true;
      end;
    end;
  Result:=ShowModal=mrOK;
  end;

procedure TFilterDialog.btnEditMaskClick(Sender: TObject);
var
  s  : string;
  p  : TPoint;
  ml : TStringList;
begin
  s:='';
  ml:=TStringList.Create;
  with ml do begin
    Delimiter:=FilterSeparator;
    DelimitedText:=cbxFilter.Text;
    end;
  with btnEditMask do p:=ClientToScreen(Point(-20,Height+5));
  if SelectFromListDialog.Execute(p,Caption,lbMask.Caption,
              true,false,false,0,tcLower,cbxFilter.Text,ml,s) then begin
    cbxFilter.Text:=ml.DelimitedText;
    ml.Free;
    end;
  end;

end.
