(*  Delphi Dialog
    FTP-Verzeichnis auswhlen
    =========================
    
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Apr. 2007
   *)
    
unit FtpDirDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, FtGlobal, FtpFileTools, ImgList;

type
  TFtpDirDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Label1: TLabel;
    edBaseDir: TEdit;
    Label2: TLabel;
    tvSubDirs: TTreeView;
    ImageList: TImageList;
    edSubDir: TEdit;
    Label3: TLabel;
    procedure tvSubDirsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    FTP    : TExtFTP;
    procedure BuildDirTree (Dir : string; Node : TTreeNode);
  public
    { Public-Deklarationen }
    function Execute (AHost,AUsername,APassword,ADirectory : string;
                      APort : integer; var ASubDir : string) : boolean;
  end;

var
  FtpDirDialog: TFtpDirDialog;

implementation

uses PwdDlg, JrUtils, StatWind, GnuGetText;

{$R *.dfm}

{------------------------------------------------------------------- }
procedure TFtpDirDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TFtpDirDialog.BuildDirTree(Dir : string; Node : TTreeNode);
var
  i    : integer;
  dl   : TStringList;
  nd   : TTreeNode;
  base,
  ok   : boolean;
begin
  dl:=TStringList.Create;
  ok:=true; base:=length(Dir)=0;
  if not base then begin
    try Ftp.ChangeDir(ExtractFilename(Dir)); except ok:=false; end;
    end;
  if ok then begin
    Ftp.GetDirList(dl);
    nd:=Node;
    with dl do for i:=0 to Count-1 do begin
      if base then nd:=tvSubDirs.Items.Add(nd,Strings[i])
      else nd:=tvSubDirs.Items.AddChild(Node,Strings[i]);
      with nd do begin
        ImageIndex:=0;
        SelectedIndex:=1;
        end;
      BuildDirTree(Strings[i],nd);
      Application.ProcessMessages;
      end;
    if not base then Ftp.ChangeDirUp;
    end;
  dl.Free;
  end;

procedure TFtpDirDialog.tvSubDirsClick(Sender: TObject);
var
  s    : string;
  nd   : TTreeNode;
begin
  nd:=TvSubDirs.Selected;
  s:=nd.Text;
  while nd.Parent<>nil do begin
    nd:=nd.Parent;
    s:=nd.Text+'/'+s;
    end;
  edSubDir.Text:=s;
  end;

{------------------------------------------------------------------- }
function TFtpDirDialog.Execute (AHost,AUsername,APassword,ADirectory : string;
                                APort : integer; var ASubDir : string) : boolean;
var
  ok : boolean;
begin
  Ftp:=TExtFtp.Create(self);
  with Ftp do begin
    Host:=AHost;
    Port:=APort;
    Username:=AUsername;
    TransferTimeout:=defTimeout;
    if length(APassword)=0 then begin
      ok:=ReadPassword(rsFtpConn,Format(rsFtpHint,[Host,Port,Username]),APassword,false)=mrOK;
      end
    else ok:=true;
    if ok then begin
      edBaseDir.Text:='ftp://'+Host+':'+IntToStr(Port)+'/'+ADirectory;
      StatusWindow.ShowStatus (CenterPos,rsFtpConn,rsTryConnect+': '+Host,true,10);
      Application.ProcessMessages;
      Password:=APassword;
      try Connect; except ok:=false; end;
      if ok then ok:=ForceDir(ADirectory)
      else ErrorDialog(Caption,rsConnectErr);
      StatusWindow.Status:=dgettext('dialogs','Scanning directories');
      tvSubDirs.Items.Clear;
      try
        BuildDirTree('',nil);
      finally
        Ftp.Disconnect;
        StatusWindow.Close;
        end;
      end;
    if ok and (ShowModal=mrOK) then begin
      ASubDir:=edSubDir.Text;
      Result:=true;
      end
    else Result:=false;
    end;
  Ftp.Free;
  end;

end.
