(* Umsetzung von Graustufen in Falschfarben (Dialog)

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Jul. 2001
   *)

unit GTCDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, NumberEd, StdCtrls, Buttons, ExtCtrls, ComCtrls, JPEG, Tiff,
  Spin, ExtDlgs, HListBox, WinMath, JrUtils;

type
  TUserPal = record
    lpal  : TLogPalette;
    dummy : Array[0..255] of TPaletteEntry;
    end;

  TRGB = record
    b,g,r : byte;
    end;
  TxPal = Array [0..255] of TRGB;

  TGTCDialog = class(TForm)
    DataGB: TGroupBox;
    StatusPanel: TPanel;
    OKBtn: TBitBtn;
    TBStart: TTrackBar;
    TBDelta: TTrackBar;
    Bevel1: TBevel;
    ImageSize: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    GrafikPanel: TPanel;
    PaintBox: TPaintBox;
    CancelBtn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure TBStartChange(Sender: TObject);
    procedure TBDeltaChange(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
  private
    { Private-Deklarationen }
    FName       : string;
    chg         : boolean;
    wbm,
    FBitmap     : TBitmap;
    UserPal     : TUserPal;
    XPal        : TXPal;

    function SetScreenScale : integer;
    procedure SetPaletteColors (var APal    : TXPal;
                                Start,Delta : integer);
    procedure Refresh;
  public
    { Public-Deklarationen }
    function Execute (AName,TName   : string;
                      const ABitmap : TBitmap): boolean;
  end;

var
  GTCDialog: TGTCDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TGTCDialog.FormCreate(Sender: TObject);
var
  i: integer;
begin
  TranslateComponent (self,'dialogs');
  with UserPal.lpal do begin
    palVersion:=$300;
    palNumEntries:=256;
    for i:=0 to 255 do with PalpalEntry[i] do begin
      peRed:=i; peGreen:=i; peBlue:=i; peFlags:=0;
      end;
    end;
  SetPaletteColors (XPal,0,0);
  with TBStart do Position:=Min;
  with TBDelta do Position:=Min;
  chg:=false;
  end;

{ ------------------------------------------------------------------- }
procedure TGTCDialog.SetPaletteColors (var APal    : TXPal;
                                       Start,Delta : integer);
var
  r,g,b,h,
  i,j      : integer;

  function ColorScaleInc (w : integer) : integer;
  begin
    Result:=round(255*(1-sqr((w-60)/60)));
    end;

  function ColorScaleDec (w : integer) : integer;
  begin
    Result:=255-round(255*sqr(w/60));
    end;

begin
  if Delta=0 then begin
    for i:=0 to 255 do with XPal[i] do begin
      r:=i; g:=i; b:=i;
      end;
    end
  else begin
    for i:= 0 to 255 do with XPal[i] do begin
      h:=round(Delta*i/255+Start) mod 360;
      if h<=60 then begin
        r:=255; b:=0;
        g:=ColorScaleInc(h);
        end
      else if h<=120 then begin
        g:=255; b:=0;
        r:=ColorScaleDec(h-60);
        end
      else if h<=180 then begin
        g:=255; r:=0;
        b:=ColorScaleInc(h-120);
        end
      else if h<=240 then begin
        b:=255; r:=0;
        g:=ColorScaleDec(h-180);
        end
      else if h<=300 then begin
        b:=255; g:=0;
        r:=ColorScaleInc(h-240);
        end
      else begin
        r:=255; g:=0;
        b:=ColorScaleDec(h-300);
        end;
      end;
    end;
 end;

{ ------------------------------------------------------------------- }
procedure TGTCDialog.Refresh;
begin
  if assigned(FBitmap) then with FBitmap do if not Empty then begin
    SetPaletteColors (XPal,TBStart.Position,TBDelta.Position);
    PaintBox.Invalidate
    end;
  end;

procedure TGTCDialog.TBStartChange(Sender: TObject);
begin
  Refresh;
  end;

procedure TGTCDialog.TBDeltaChange(Sender: TObject);
begin
  Refresh;
  end;

{ ------------------------------------------------------------------- }
function TGTCDialog.SetScreenScale : integer;
begin
  with PaintBox,Canvas do begin
    Result:=SetMapMode (Handle,MM_ISOTROPIC);
    SetWindowOrgEx (Handle,0,0,nil);
    SetWindowExtEx (Handle,FBitmap.Width,FBitmap.Height,nil);
{    SetViewPortOrg (Handle,0,0); darf hier nicht stehen, da sonst OnPaint fehlerhaft }
    SetViewportExtEX (handle,Width,Height,nil);
    end;
  end;

procedure TGTCDialog.PaintBoxPaint(Sender: TObject);
var
  P     : PByteArray;
  RGB   : ^TRGB;
  i,j   : integer;
begin
  if assigned(FBitmap) and not FBitmap.Empty then begin
    SetScreenScale;
    with FBitmap do begin
      for j:=0 to Height-1 do begin
        P := ScanLine[j];
        RGB:=wbm.ScanLine[j];
        for i:=0 to Width-1 do with XPal[P[i]]do begin
          RGB^.r:=r;
  //        RGB^.r:=0; RGB^.b:=0;
          RGB^.g:=g;
          RGB^.b:=b;
          inc(RGB);
          end;
        end;
      PaintBox.Canvas.StretchDraw(Rect(0,Height,Width,0),wbm);
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
function TGTCDialog.Execute (AName,TName   : string;
                             const ABitmap : TBitmap) : Boolean;
var
  w : integer;
begin
  FName:=AName; FBitmap:=ABitmap;
  Caption:=StripPath(FName,45);
  with FBitmap do ImageSize.Caption:=Strint(Width,5)+', '+Strint(Height,5);
  if assigned (wbm) then wbm.Free;
  wbm:=TBitMap.Create;
  with wbm do begin
    Pixelformat:=pf24bit;
    Height:=FBitmap.Height; Width:=FBitmap.Width;
    end;
  with FBitmap do begin
    w:=round((ClientWidth-2)/Width*Height);
    if w<3*Screen.Height div 4 then ClientHeight:=w+2+StatusPanel.Height+DataGb.Height;
    end;
  if ShowModal=mrOK then begin
    wbm.SaveToFile(TName);
    wbm.Free; wbm:=nil;
    Result:=true;
    end
  else Result:=false;
  end;

end.
