(* Delphi Dialog
   Feiertage eines Jahr verwalten
   ==============================
   (siehe auch EditHolidayDlg)

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jun. 2004
   *)

unit HolidayDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, DateUtilEx, CheckLst;

const
  AutoStart = 1;
  SysTray = 2;

type
  THolidayDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    cbxAutostart: TCheckBox;
    cbxSystray: TCheckBox;
    GroupBox2: TGroupBox;
    clbHolidays: TCheckListBox;
    btnEdit: TBitBtn;
    Image1: TImage;
    procedure btnEditClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FHolidayCalendar : THolidayCalendar;
    procedure ShowHolidays;
  public
    { Public declarations }
    function Execute (var AHolidayCalendar : THolidayCalendar;
                      var Options          : integer) : boolean;
  end;

var
  HolidayDialog: THolidayDialog;

implementation

uses EditHolidayDlg, GnuGetText;

{$R *.DFM}

{------------------------------------------------------------------- }
procedure THolidayDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure THolidayDialog.ShowHolidays;
var
  i : integer;
begin
  with FHolidayCalendar do begin
    clbHolidays.Clear;
    for i:=0 to HolidayCount-1 do with clbHolidays do
        Checked[Items.AddObject(Holiday[i].Name,pointer(i))]:=Holiday[i].Active;
    end;
  end;

{------------------------------------------------------------------- }
(* Dialog anzeigen *)
function THolidayDialog.Execute (var AHolidayCalendar : THolidayCalendar;
                                 var Options          : integer) : boolean;
var
  i,n   : integer;
  h     : THoliday;
begin
  FHolidayCalendar:=AHolidayCalendar;
  ShowHolidays;
  cbxAutostart.Checked:=Options and Autostart <>0;
  cbxSystray.Checked:=Options and SysTray <>0;
  if ShowModal=mrOK then with FHolidayCalendar do begin
    with clbHolidays,Items do for i:=0 to Count-1 do begin
      n:=integer(Items.Objects[i]);
      h:=Holiday[n];
      h.Active:=Checked[i];
      Holiday[n]:=h;
      end;
    if cbxAutostart.Checked then Options:=Options or Autostart
    else Options:=Options and not Autostart;
    if cbxSystray.Checked then Options:=Options or SysTray
    else Options:=Options and not SysTray;
    Result:=true;
    end
  else Result:=false;
  end;

procedure THolidayDialog.btnEditClick(Sender: TObject);
var
  s : string;
begin
  with clbHolidays do if ItemIndex>=0 then s:=Items[ItemIndex] else s:='';
  if EditHolidayDialog.Execute (FHolidayCalendar,s) then ShowHolidays;
  end;

end.
