(*  Dialog Zahleneingabe von 2 Werten (Fliekomme)
    - Min- und Maxwerte einstellbar
    - Format einstellbar
    - Anzahl der Stellen einstellbar
    - opt. Feld fr Einheiten

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Sep. 2002
   *)

unit InpDFloat;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, NumberEd, WinMath, JrUtils;

type
  TInputDFloatDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Desc1Label: TLabel;
    Float1Edit: TFloatRangeEdit;
    Unit1Label: TLabel;
    Float2Edit: TFloatRangeEdit;
    Desc2Label: TLabel;
    Unit2Label: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

(* show dialog and get value *)
function InputDFloat(Pos              : TPoint;
                    Titel,
                    Desc1,AUnits1,
                    Desc2,AUnits2,
                    ACancel          : string;
                    AFormat          : TFloatFormat;
                    ADigits1,
                    ADecimal1,
                    ADigits2,
                    ADecimal2        : integer;
                    Min1,Max1,
                    Min2,Max2        : double;
                    var Val1,Val2    : double) : boolean;

var
  InputDFloatDialog : TInputDFloatDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TInputDFloatDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TInputDFloatDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s  : string;
  ok : boolean;
  p  : TPoint;
begin
  ok:=CanClose;
  with Float1Edit do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if FloatFormat=ffNormalized then
        s:=FloatToStrP(MinValue,DecimalSeparator,Digits)+
                      ' .. '+FloatToStrP(MaxValue,DecimalSeparator,Digits)+')'
      else s:=FloatToStrF(MinValue,TFloatFormat(FloatFormat),Digits,Decimal)+' .. '+
         FloatToStrF(MaxValue,TFloatFormat(FloatFormat),Digits,Decimal)+')';
      with ClientOrigin do p:=Point(X+Width,Y+Height);
      CanClose:=false;
      end;
    end;
  if ok then   with Float1Edit do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if FloatFormat=ffNormalized then
        s:=FloatToStrP(MinValue,DecimalSeparator,Digits)+
                      ' .. '+FloatToStrP(MaxValue,DecimalSeparator,Digits)+')'
      else s:=FloatToStrF(MinValue,TFloatFormat(FloatFormat),Digits,Decimal)+' .. '+
         FloatToStrF(MaxValue,TFloatFormat(FloatFormat),Digits,Decimal)+')';
      with ClientOrigin do p:=Point(X+Width,Y+Height);
      CanClose:=false;
      end;
    end;
  CanClose:=ok;
  if not ok then
  ErrorDialog('','Error: Out of range ('+s,p);
  end;

{ ------------------------------------------------------------------- }
(* show dialog and get value *)
function InputDFloat(Pos              : TPoint;
                    Titel,
                    Desc1,AUnits1,
                    Desc2,AUnits2,
                    ACancel          : string;
                    AFormat          : TFloatFormat;
                    ADigits1,
                    ADecimal1,
                    ADigits2,
                    ADecimal2        : integer;
                    Min1,Max1,
                    Min2,Max2        : double;
                    var Val1,Val2    : double) : boolean;
begin
  if not assigned(InputDFloatDialog) then
    InputDFloatDialog:=TInputDFloatDialog.Create(Application);
  with InputDFloatDialog do begin
    Left:=Pos.x; Top:=Pos.y;
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    Desc1Label.Caption:=Desc1;
    Unit1Label.Caption:=AUnits1;
    with Float1Edit do begin
      FloatFormat:=TNumFloatFormat(AFormat);
      Digits:=ADigits1; Decimal:=ADecimal1;
      MinValue:=DecimalRound(Min1,ADecimal1); MaxValue:=DecimalRound(Max1,ADecimal1);
      RangeCheck:=false;
      Value:=Val1;
      end;
    Desc2Label.Caption:=Desc2;
    Unit2Label.Caption:=AUnits2;
    with Float2Edit do begin
      FloatFormat:=TNumFloatFormat(AFormat);
      Digits:=ADigits2; Decimal:=ADecimal2;
      MinValue:=DecimalRound(Min2,ADecimal2); MaxValue:=DecimalRound(Max2,ADecimal2);
      RangeCheck:=false;
      Value:=Val2;
      end;
    ActiveControl:=Float1Edit;
    if ShowModal=mrOK then begin
      Val1:=Float1Edit.Value;
      Val2:=Float2Edit.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputDFloatDialog:=nil;
  end;

end.
