(* Delphi-Unit
   Zahleneingabefenster fr zwei Intergerzahlen (dez. oder hex) mit Spin-Tasten

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Mai 2002
   *)


unit InpDNumber;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, NumberEd, Spin, JrUtils;

type
  TDInputNumberDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor1: TLabel;
    SpinButton1: TSpinButton;
    Units1: TLabel;
    NumFeld1: TRangeEdit;
    Descriptor2: TLabel;
    NumFeld2: TRangeEdit;
    SpinButton2: TSpinButton;
    Units2: TLabel;
    procedure SpinButton1DownClick(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SpinButton2UpClick(Sender: TObject);
    procedure SpinButton2DownClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FIncr1,FIncr2 : integer;
  public
    { Public declarations }
  end;

(* Zahl eingeben, Ergebnis: "true" bei "ok" *)
function InputDInteger(Pos           : TPoint;
                       Titel,ACancel,
                       Desc1,AUnits1 : string;
                       AIncr1,
                       ADigits1,
                       Min1,Max1     : integer;
                       AHex1         : boolean;
                       Desc2,AUnits2 : string;
                       AIncr2,
                       ADigits2,
                       Min2,Max2     : integer;
                       AHex2         : boolean;
                       var Val1,Val2 : integer) : boolean;

var
  DInputNumberDialog: TDInputNumberDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TDInputNumberDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TDInputNumberDialog.SpinButton1DownClick(Sender: TObject);
begin
  with NumFeld1 do begin
    Value:=Value-FIncr1;
    if Value<MinValue then Value:=MinValue;
    if Value>MaxValue then Value:=MaxValue;
    end;
  end;

procedure TDInputNumberDialog.SpinButton1UpClick(Sender: TObject);
begin
  with NumFeld1 do begin
    Value:=Value+FIncr1;
    if Value>MaxValue then Value:=MaxValue;
    if Value<MinValue then Value:=MinValue;
    end;
  end;

procedure TDInputNumberDialog.SpinButton2DownClick(Sender: TObject);
begin
  with NumFeld2 do begin
    Value:=Value-FIncr2;
    if Value<MinValue then Value:=MinValue;
    if Value>MaxValue then Value:=MaxValue;
    end;
  end;

procedure TDInputNumberDialog.SpinButton2UpClick(Sender: TObject);
begin
  with NumFeld2 do begin
    Value:=Value+FIncr2;
    if Value>MaxValue then Value:=MaxValue;
    if Value<MinValue then Value:=MinValue;
    end;
  end;

procedure TDInputNumberDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s : string;
begin
  with NumFeld1 do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)+')'
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue)+')';
      with ClientOrigin do ErrorDialog ('',Format(_'Error: Out of Range (%s)'),[s],Point(X+Width,Y+Height));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  if CanClose then with NumFeld2 do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)+')'
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue)+')';
      with ClientOrigin do ErrorDialog ('',Format(dgettext('dialogs','Error: Out of Range (%s)'),[s],Point(X+Width,Y+Height));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  end;

{ ------------------------------------------------------------------- }
(* 2 Zahlen eingeben, Ergebnis: "true" bei "ok" *)
function InputDInteger(Pos        : TPoint;
                       Titel,ACancel,
                       Desc1,AUnits1 : string;
                       AIncr1,
                       ADigits1,
                       Min1,Max1     : integer;
                       AHex1         : boolean;
                       Desc2,AUnits2 : string;
                       AIncr2,
                       ADigits2,
                       Min2,Max2     : integer;
                       AHex2         : boolean;
                       var Val1,Val2 : integer) : boolean;
begin
  if not assigned(DInputNumberDialog) then
    DInputNumberDialog:=TDInputNumberDialog.Create(Application);
  with DInputNumberDialog do begin
    Left:=Pos.x; Top:=Pos.y;
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    Descriptor1.Caption:=Desc1;
    Units1.Caption:=AUnits1;
    FIncr1:=AIncr1;
    with NumFeld1 do begin
      MinValue:=Min1; MaxValue:=Max1;
      Digits:=ADigits1; Hex:=AHex1; RangeCheck:=false;
      Value:=Val1;
      end;
    Descriptor2.Caption:=Desc2;
    Units2.Caption:=AUnits2;
    FIncr2:=AIncr2;
    with NumFeld2 do begin
      MinValue:=Min2; MaxValue:=Max2;
      Digits:=ADigits2; Hex:=AHex2; RangeCheck:=false;
      Value:=Val2;
      end;
    ActiveControl:=NumFeld1;
    if ShowModal=mrOK then begin
      Val1:=NumFeld1.Value;
      Val2:=NumFeld2.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  DInputNumberDialog:=nil;
  end;

end.
