(*  Dialog Zahleneingabe (Fliekomme)
    - Min- und Maxwerte einstellbar
    - Format einstellbar
    - Anzahl der Stellen einstellbar
    - opt. Feld fr Einheiten

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Sep. 2002
   *)

unit InpFloat;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, NumberEd, JrUtils, WinMath;

type
  TInputFloatDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    DescLabel: TLabel;
    FloatEdit: TFloatRangeEdit;
    UnitLabel: TLabel;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

(* show dialog and get value *)
function InputFloat(Pos              : TPoint;
                    Titel,Desc,AUnits,
                    ACancel          : string;
                    AFormat          : TFloatFormat;
                    ADigits,ADecimal : integer;
                    Min,Max          : double;
                    var Val          : double) : boolean;

var
  InputFloatDialog : TInputFloatDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TInputFloatDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TInputFloatDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s : string;
begin
  with FloatEdit do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if FloatFormat=ffNormalized then
        s:=FloatToStrP(MinValue,DecimalSeparator,Digits)+
                      ' .. '+FloatToStrP(MaxValue,DecimalSeparator,Digits)+')'
      else s:=FloatToStrF(MinValue,TFloatFormat(FloatFormat),Digits,Decimal)+' .. '+
         FloatToStrF(MaxValue,TFloatFormat(FloatFormat),Digits,Decimal)+')';
      with ClientOrigin do ErrorDialog ('','Error: Out of range ('+s,Point(X+Width,Y+Height));
      CanClose:=false;
      end
    else CanClose:=true;
    end;
  end;

{ ------------------------------------------------------------------- }
(* show dialog and get value *)
function InputFloat(Pos              : TPoint;
                    Titel,Desc,AUnits,
                    ACancel          : string;
                    AFormat          : TFloatFormat;
                    ADigits,ADecimal : integer;
                    Min,Max          : double;
                    var Val          : double) : boolean;
begin
  if not assigned(InputFloatDialog) then InputFloatDialog:=TInputFloatDialog.Create(Application);
  with InputFloatDialog do begin
    Left:=Pos.x; Top:=Pos.y;
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    DescLabel.Caption:=Desc;
    UnitLabel.Caption:=AUnits;
    with FloatEdit do begin
      FloatFormat:=TNumFloatFormat(AFormat);
      Digits:=ADigits; Decimal:=ADecimal;
      MinValue:=DecimalRound(Min,ADecimal); MaxValue:=DecimalRound(Max,ADecimal);
      RangeCheck:=false;
      Value:=Val;
      end;
    ActiveControl:=FloatEdit;
    if ShowModal=mrOK then begin
      Val:=FloatEdit.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputFloatDialog:=nil;
  end;

end.
