(* Delphi Dialog
   Eingabe und Auswertung eines mathematischen Ausdrucks
   =====================================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Sep. 2002
   *)

unit InpMath;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, HListBox;

type
  TInputMathExpDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor: TLabel;
    edtExp: THistoryCombo;
    procedure edtExpKeyPress(Sender: TObject; var Key: Char);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FValue : double;
  public
    { Public declarations }
  (* History-Liste laden (optional) *)
  procedure LoadFromIni(AIniName,AIniSection : string);
  function Execute (Pos        : TPoint;
                    Titel,Desc : string;
                    var AValue : double) : boolean;
  end;

(* Text eingeben, Ergebnis: "true" bei "ok" *)
function InputMathExpression(Pos        : TPoint;
                             Titel,Desc : string;
                             var AValue : double) : boolean;

var
  InputMathExpDialog: TInputMathExpDialog;

implementation

{$R *.DFM}

uses JrUtils, Math, WinMath, MathExp, GnuGetText;

{ ------------------------------------------------------------------- }
procedure TInputMathExpDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TInputMathExpDialog.LoadFromIni(AIniName,AIniSection : string);
begin
  with edtExp do begin
    LoadFromIni(AIniName,AIniSection );
    if Items.Count=0 then Style:=csSimple else Style:=csDropDown;
    end;
  end;

{ ------------------------------------------------------------------- }
function TInputMathExpDialog.Execute (Pos        : TPoint;
                                      Titel,Desc : string;
                                      var AValue : double) : boolean;
begin
  Left:=Pos.x; Top:=Pos.y;
  Caption:=Titel;
  Descriptor.Caption:=Desc;
  ActiveControl:=edtExp;
  with edtExp do if AValue>MinDouble then Text:=FloatToStrP(AValue,DecimalSeparator,5)
  else Text:='';
  if ShowModal=mrOK then begin
    AValue:=FValue;
    with edtExp do begin
      AddItem(Text); Style:=csDropDown;
      end;
    Result:=true;
    end
  else Result:=false;
  end;

procedure TInputMathExpDialog.edtExpKeyPress(Sender: TObject;
  var Key: Char);
begin
  if (Key = #13) then ModalResult:=mrOK
  else if (Key=#$1b) then ModalResult:=mrCancel;
  end;

procedure TInputMathExpDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if Modalresult=mrOK then begin
    CanClose:=Calculate(ReplaceDecSeps(edtExp.Text),FValue);
    if not CanClose then begin
      ErrorDialog(Caption,CalcErrorText);
      with edtExp do begin
        SelStart:=CalcErrorPos-1; SelLength:=CalcErrorLength;
        SetFocus;
        end;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Txt eingeben, Ergebnis: "true" bei "ok" *)
function InputMathExpression(Pos        : TPoint;
                   Titel,Desc : string;
                   var AValue : double) : boolean;
begin
  if not assigned(InputMathExpDialog) then
    InputMathExpDialog:=TInputMathExpDialog.Create(Application);
  with InputMathExpDialog do begin
    Result:=Execute(Pos,Titel,Desc,AValue);
    Release;
    end;
  InputMathExpDialog:=nil;
  end;

end.
