(* Delphi Dialog
   Erstellen und bearbeiten von Listen  
   ===================================

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Okt. 1999
   Vers. 1.1 - Okt. 2004: keine Beschrnkung auf 20 Eintrge mehr
   *)
      
unit Listdlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  SysUtils, StdCtrls, Grids;

const
  DefRowCount = 21;

type
  TConvertMode = (cvLower,cvUpper);
  TConvertModes = set of TConvertMode;

  TListenDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Liste: TStringGrid;
    NameEdit: TEdit;
    Label2: TLabel;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    DeleteBtn: TSpeedButton;
    InsertBtn: TSpeedButton;
    ReplBtn: TSpeedButton;
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure ListeSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure DeleteBtnClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure ReplBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    LineCount : integer;
  public
    { Public declarations }
    FCvMode   : TConvertModes;
    function Execute (Title,Desc        : string;
                      APos              : TPoint;
                      CvMode            : TConvertModes;
                      var AListe        : string) : boolean;
  end;

var
  ListenDialog: TListenDialog;

implementation

{$R *.DFM}

uses JrUtils, GnuGetText;

{------------------------------------------------------------------- }
procedure TListenDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  FcvMode:=[];
  end;

procedure TListenDialog.UpBtnClick(Sender: TObject);
begin
  with Liste do if (Row>1) and (Row<=LineCount) then begin
    Cols[1].Exchange(Row,Row-1);
    Row:=Row-1;
    NameEdit.Text:=cells[1,Row];
    end;
  NameEdit.Selectall;
  end;

procedure TListenDialog.DownBtnClick(Sender: TObject);
begin
  with Liste do if Row<LineCount then begin
    Cols[1].Exchange(Row,Row+1);
    Row:=Row+1;
    NameEdit.Text:=cells[1,Row];
    end;
  NameEdit.Selectall;
  end;

procedure TListenDialog.DeleteBtnClick(Sender: TObject);
var
  i : integer;
begin
  if LineCount>0 then begin
    dec(LineCount);
    with Liste do begin
      for i:=Row to Linecount do begin
        cells[1,i]:=cells[1,succ(i)];
        end;
      cells[1,succ(LineCount)]:='';
      if (Row=succ(LineCount)) and (Row>1) then Row:=Row-1;
      end;
    end;
  NameEdit.Selectall;
  end;

procedure TListenDialog.InsertBtnClick(Sender: TObject);
var
  i,n : integer;
begin
  with Liste do begin
    if LineCount=pred(RowCount) then begin
      RowCount:=RowCount+1;
      cells[0,RowCount-1]:=IntToStr(RowCount-1);
      end;
    if Row=LineCount then begin   // letzte Zeile, neuen Wert anhngen
      n:=Row+1;
      end
    else begin
      for i:=Linecount downto Row do cells[1,succ(i)]:=cells[1,i];
      n:=Row;
      end;
    if (cvLower in FCvMode) then cells[1,n]:=AnsiLowerCase(NameEdit.Text)
    else if (cvUpper in FCvMode) then cells[1,n]:=AnsiUpperCase(NameEdit.Text)
    else cells[1,n]:=NameEdit.Text;
    inc(LineCount);
    if n>Row then Row:=n;
    end;
  NameEdit.Selectall;
  end;

procedure TListenDialog.ListeSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
  with Liste do if Row=RowCount-1 then RowCount:=RowCount+1;
  CanSelect:=Row<=LineCount;
  if CanSelect then NameEdit.Text:=Liste.cells[1,Row];
  if Active then NameEdit.SetFocus;
  end;

procedure TListenDialog.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then InsertBtnClick(Sender);
  end;

procedure TListenDialog.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$0D then Key:=#0;
  end;

procedure TListenDialog.ReplBtnClick(Sender: TObject);
begin
  with Liste do
    if (cvLower in FCvMode) then cells[1,Row]:=AnsiLowerCase(NameEdit.Text)
    else if (cvUpper in FCvMode) then cells[1,Row]:=AnsiUpperCase(NameEdit.Text)
    else cells[1,Row]:=NameEdit.Text;
  NameEdit.Selectall;
  end;

{------------------------------------------------------------------- }
function TListenDialog.Execute (Title,Desc        : string;
                                APos              : TPoint;
                                CvMode            : TConvertModes;
                                var AListe        : string) : boolean;
var
  ok : boolean;
  i,j : integer;
  s   : string;
begin
  with APos do begin
    if (Y < 0) or (X < 0) then Position:=poScreenCenter
    else begin
      if X<0 then X:=Left;
      if Y<0 then Y:=Top;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    end;
  if Title='' then Caption:=dgettext('dialogs','Edit list') else Caption:=Title;
  FCvMode:=CvMode;
  s:=AListe;
  with Liste do begin
    Cols[1].Clear;
    ColWidths[0]:=25; ColWidths[1]:=135;
    RowCount:=DefRowCount;
    cells[0,0]:=dgettext('dialogs','No.'); cells[1,0]:=Desc;
    NameEdit.Text:='';
    j:=1;
    if length(s)>0 then repeat
      i:=pos(',',s);
      if i=0 then i:=succ(length(s));
      cells[1,j]:=copy(s,1,pred(i));
      delete (s,1,i);
      inc(j);
      if j>=RowCount then RowCount:=RowCount+1;
      until (length(s)=0);
    for i:=1 to RowCount-1 do cells[0,i]:=IntToStr(i);
    Row:=1;
    NameEdit.Text:=cells[1,Row];
    end;
  LineCount:=pred(j);
  ActiveControl:=NameEdit;
  ok:=ShowModal=mrOK;
  if ok then begin
    s:='';
    with Liste do for i:=1 to LineCount do s:=s+cells[1,i]+',';
    delete (s,length(s),1);
    AListe:=s;
    end;
  Result:=ok;  
  end;

end.
