(* Delphi-Dialog
   Auswahl eines Drucker-Ports (1 .. 4) oder individ. Angabe von Port und Irq

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Mai 2002
   *)

unit LptSelect;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, NumberEd, Spin, JrUtils;

const
  (* Standard-LPT *)
  LPTPortMax = 4;
  LPTPort: array [1..LPTPortMax] of word = ($378,$278,$3BC,$2BC);
  LPTIRQ : array [1..LPTPortMax] of byte = (7,5,3,9);

type
  TInputLptPortDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    LptSel: TRadioGroup;
    Panel1: TPanel;
    Descriptor1: TLabel;
    SpinButton1: TSpinButton;
    PortEdit: TRangeEdit;
    Descriptor2: TLabel;
    IrqEdit: TRangeEdit;
    SpinButton2: TSpinButton;
    procedure SpinButton1DownClick(Sender: TObject);
    procedure SpinButton1UpClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure SpinButton2UpClick(Sender: TObject);
    procedure SpinButton2DownClick(Sender: TObject);
    procedure LptSelClick(Sender: TObject);
    procedure PortChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    AllowChange : boolean;
  public
    { Public declarations }
  end;

(* Zahl eingeben, Ergebnis: "true" bei "ok" *)
function InputLptPort(Pos           : TPoint;
                      Titel,ACancel : string;
                      var LptNr,PortAdr,PortIrq : integer) : boolean;

var
  InputLptPortDialog: TInputLptPortDialog;

implementation

{$R *.DFM}

uses GnuGetText;

procedure TInputLptPortDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{ ------------------------------------------------------------------- }
procedure TInputLptPortDialog.SpinButton1DownClick(Sender: TObject);
begin
  with PortEdit do begin
    Value:=Value-8;
    if Value<MinValue then Value:=MinValue;
    if Value>MaxValue then Value:=MaxValue;
    end;
  end;

procedure TInputLptPortDialog.SpinButton1UpClick(Sender: TObject);
begin
  with PortEdit do begin
    Value:=Value+8;
    if Value>MaxValue then Value:=MaxValue;
    if Value<MinValue then Value:=MinValue;
    end;
  end;

procedure TInputLptPortDialog.SpinButton2DownClick(Sender: TObject);
begin
  with IrqEdit do begin
    Value:=Value-1;
    if Value<MinValue then Value:=MinValue;
    if Value>MaxValue then Value:=MaxValue;
    end;
  end;

procedure TInputLptPortDialog.SpinButton2UpClick(Sender: TObject);
begin
  with IrqEdit do begin
    Value:=Value+1;
    if Value>MaxValue then Value:=MaxValue;
    if Value<MinValue then Value:=MinValue;
    end;
  end;

procedure TInputLptPortDialog.LptSelClick(Sender: TObject);
begin
  with LptSel do if ItemIndex<4 then begin
    AllowChange:=false;
    PortEdit.Value:=LptPort[Itemindex+1];
    IrqEdit.Value:=LptIrq[Itemindex+1];
    AllowChange:=true;
    PortChange(Sender);
    end;
  end;

procedure TInputLptPortDialog.PortChange(Sender: TObject);
var
  i   : integer;
begin
  if AllowChange then begin
    i:=1;
    while (i<=LptPortMax) and ((PortEdit.Value<>LptPort[i]) or (IrqEdit.Value<>LptIrq[i])) do inc(i);
    with LptSel do if i<=LptPortMax then ItemIndex:=i-1 else ItemIndex:=4
    end;
  end;

procedure TInputLptPortDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s : string;
begin
  with PortEdit do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue);
      with ClientOrigin do ErrorDialog ('',Format(_('Error: Out of range (%s)'),[s]),Point(X+Width,Y+Height));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  if CanClose then with IrqEdit do begin
    if (ModalResult=mrOK) and (CheckValue(Value)<>Value) then begin
      if Digits>0 then s:='$'+IntToHex(MinValue,Digits)+' , $'+IntToHex(MaxValue,Digits)
      else s:=IntToStr(MinValue)+' .. '+IntToStr(MaxValue);
      with ClientOrigin do ErrorDialog ('',Format(_('Error: Out of range (%s)'),[s]),Point(X+Width,Y+Height));
      CanClose:=false;
      SetFocus;
      end
    else CanClose:=true;
    end;
  end;

{ ------------------------------------------------------------------- }
(* LPT-Port auswhlen *)
function InputLptPort(Pos           : TPoint;
                      Titel,ACancel : string;
                      var LptNr,PortAdr,PortIrq : integer) : boolean;
begin
  if not assigned(InputLptPortDialog) then
    InputLptPortDialog:=TInputLptPortDialog.Create(Application);
  with InputLptPortDialog do begin
    Left:=Pos.x; Top:=Pos.y;
    Caption:=Titel;
    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    with LptSel do if (LptNr<1) or (LptNr>LptPortMax) then ItemIndex:=4
    else begin
      if (LptPort[LptNr]=PortAdr) and (LptIrq[LptNr]=PortIrq) then ItemIndex:=LptNr-1
      else ItemIndex:=4;
      end;
    AllowChange:=true;
    with PortEdit do begin
      RangeCheck:=false; Value:=PortAdr;
      end;
    with IrqEdit do begin
      RangeCheck:=false; Value:=PortIrq;
      end;
    ActiveControl:=PortEdit;
    if ShowModal=mrOK then begin
      with LptSel do if ItemIndex=4 then LptNr:=0 else LptNr:=ItemIndex+1;
      PortAdr:=PortEdit.Value;
      PortIrq:=IrqEdit.Value;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputLptPortDialog:=nil;
  end;

end.
