(*  Delphi Dialog
    Hinweis mit Fortschrittsanzeige
    ===============================
    
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Dez. 2006
   *)

unit MsgWind;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls,
  JrUtils, Gauges, Buttons, ComCtrls;

const
  Rand = 10;
  Abstand = 5;

type
  TWinPos = (poLeftTop,poCenterTop,poRightTop,
             poLeftCenter,poCenter,poRightCenter,
             poLeftBottom,poCenterBottom,poRightBottom);

  THinweis = class(TForm)
    Rahmen: TBevel;
    MsgDesc: TLabel;
    MsgPrompt: TLabel;
    MsgSuffix: TLabel;
    MsgText: TLabel;
    Balken: TProgressBar;
    procedure HinweisCreate(Sender: TObject);

  private
    { Private-Deklarationen }
    Procedure WriteCaption (Index : Integer;
                            Text  : string);
    (* Fenster anpassen *)
    procedure InitForm (S            : string;
                        W            : integer;
                        SAlign       : TAlignment;
                        ProVal       : integer;
                        Desc,Suffix,
                        Titel,Prompt : String);

  public
    { Public-Deklarationen }
    property Descriptor : string index 1 write WriteCaption;
    property Suffix     : string index 2 write WriteCaption;
    property Prompt     : string index 3 write WriteCaption;
    property Text       : string index 4 write WriteCaption;

    (* String in Hinweisfenster bei X,Y anzeigen *)
    procedure ShowStringPos (S            : string;
                             W            : integer;
                             SAlign       : TAlignment;
                             ProVal       : integer;
                             Desc,Suffix,
                             Titel,Prompt : String;
                             X,Y          : integer);

    (* String in Hinweisfenster an gewhlter Position anzeigen *)
    procedure ShowString (S            : string;
                          W            : integer;
                          SAlign       : TAlignment;
                          ProVal       : integer;
                          Desc,Suffix,
                          Titel,Prompt : String;
                          WinPos       : TWinPos);

    (* Fortschrittsanzeige aktualisieren *)
    procedure ShowProgress (ProVal : integer);

    (* Integerzahl in Hinweisfenster bei X,Y anzeigen *)
    procedure ShowIntegerPos (N            : longint;
                              Nl           : word;
                              W            : integer;
                              Desc,Einh,
                              Titel,Prompt : String;
                              X,Y          : integer);

    (* Integerzahl in Hinweisfenster an gewhlter Position anzeigen *)
    procedure ShowInteger (N            : longint;
                           Nl           : word;
                           W            : integer;
                           Desc,Einh,
                           Titel,Prompt : String;
                           WinPos       : TWinPos);

  end;

var
  Hinweis: THinweis;

implementation

{$R *.DFM}

uses GnuGetText;

{ ---------------------------------------------------------------- }
(* Position eines Fensters auf dem Bildschirm berechnen *)
function GetWindowPos (Screen         : TScreen;
                       AWidth,AHeight : integer;
                       Position       : TWinPos) : TPoint;
(* W,H      : Breite und Hhe des anzuzeigenden Fensters
   Position : siehe TWinPos *)
var
  Org : TPoint;
begin
  with Screen do case Position of
  poCenterTop    : begin
                   Org.X:=Width div 2;
                   Org.Y:=Height div 4;
                   end;
  poCenter       : begin
                   Org.X:=Width div 2;
                   Org.Y:=Height div 2;
                   end;
  poCenterBottom : begin
                   Org.X:=Width div 2;
                   Org.Y:=3*Height div 4;
                   end;
  poLeftTop      : begin
                   Org.X:=Width div 4;
                   Org.Y:=Height div 4;
                   end;
  poLeftCenter   : begin
                   Org.X:=Width div 4;
                   Org.Y:=Height div 2;
                   end;
  poLeftBottom   : begin
                   Org.X:=Width div 4;
                   Org.Y:=3*Height div 4;
                   end;
  poRightTop     : begin
                   Org.X:=3*Width div 4;
                   Org.Y:=Height div 4;
                   end;
  poRightCenter  : begin
                   Org.X:=3*Width div 4;
                   Org.Y:=Height div 2;
                   end;
  poRightBottom  : begin
                   Org.X:=3*Width div 4;
                   Org.Y:=3*Height div 4;
                   end;
    end;
  with Org do begin
    X:=X-AWidth div 2; Y:=Y-AHeight div 2;
    CheckScreenBounds (Screen,X,Y,AWidth,AHeight);
    end;
  Result:=Org;
  end;

{ ---------------------------------------------------------------- }
procedure THinweis.HinweisCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  Descriptor:=''; Suffix:=''; Prompt:='';
  end;

{ ---------------------------------------------------------------- }
(* Fenster anpassen *)
procedure THinweis.InitForm  (S            : string;
                              W            : integer;
                              SAlign       : TAlignment;
                              ProVal       : integer;
                              Desc,Suffix,
                              Titel,Prompt : String);
var
  i,wp : integer;
  Neu  : boolean;
begin
  Neu:=not Visible;
  if Neu or (Titel<>'') then Caption:=Titel;
  with MsgDesc do begin
    if Desc<>'' then Caption:=Desc;
    if Neu then begin
      Left:=2*Rand;
      if length(Desc)>0 then i:=Left+Width+Abstand
      else i:=Left;
      end;
    end;
  with MsgText do begin
    Caption:=s; Alignment:=SAlign;
    if Neu then begin
      if W>0 then begin
        AutoSize:=false; Width:=W
        end
      else AutoSize:=true;
      Left:=i;
      i:=Left+Width;
      end;
    end;
  with MsgSuffix do begin
    if Suffix<>'' then Caption:=Suffix;
    if Neu then begin
      if length(Desc)>0 then begin
        Left:=i+Abstand;
        i:=Left+Width;
        end
      else Left:=i;
      end;
    end;
  with MsgPrompt do begin
    AutoSize:=true;
    Align:=alNone; Top:=2*Rand+Rahmen.Height;
    Left:=Rand;
    if Neu or (Prompt<>'') then Caption:=Prompt;
    wp:=Width;
    AutoSize:=false;
    end;
  if Neu then with Rahmen do begin
    Left:=Rand;
    if i>wp then Width:=i else Width:=wp;
    end;
  with Balken do if ProVal>=0 then begin
    Position:=ProVal;
    if Neu then begin
      Top:=MsgPrompt.Top+MsgPrompt.Height+Rand;
      Left:=Rahmen.Left;
      Width:=Rahmen.Width;
      Visible:=true;
      end;
    end
  else if Neu then begin
    Visible:=false;
    end;
  if Neu then begin
    MsgPrompt.Width:=Rahmen.Width;
    if Balken.Visible then ClientHeight:=Balken.Top+Balken.Height+Rand
    else ClientHeight:=MsgPrompt.Top+MsgPrompt.Height+Rand;
    Width:=Rahmen.Width+3*Rand;
    end;
  end;

Procedure THinweis.WriteCaption (Index : Integer;
                                 Text  : string);
begin
  case Index of
  1 : MsgDesc.Caption:=Text;
  2 : MsgSuffix.Caption:=Text;
  3 : MsgPrompt.Caption:=Text;
  4 : MsgText.Caption:=Text;
    end;
  end;

{ ---------------------------------------------------------------- }
(* String in Hinweisfenster bei X,Y anzeigen *)
procedure THinweis.ShowStringPos (S            : string;
                                  W            : integer;
                                  SAlign       : TAlignment;
                                  ProVal       : integer;
                                  Desc,Suffix,
                                  Titel,Prompt : String;
                                  X,Y          : integer);
(* S      : anzuzeigender String
   W      : Breite fr S (0 = anpassen)
   SAlign : Ausrichtung des Strings
   ProVal : Wert Fortschrittsanzeige (0..100%) (< 0: aus)
   Desc   : dem String vorangestellter Text
   Suffix : dem String nachfolgender Text
   Titel  : Fensterberschrift
   Prompt : Hinweis
   X,Y    : Position der linken oberen Fensterecke *)
begin
  InitForm (S,W,SAlign,ProVal,Desc,Suffix,Titel,Prompt);
  if not visible then begin
    if X>=0 then begin
      CheckScreenBounds (Screen,X,Y,Width,Height);
      Left:=X; Top:=Y;
      Position:=poDesigned;
      end
    else Position:=poScreenCenter;
    Show;
    end
  else Update;
  end;

{ ---------------------------------------------------------------- }
(* String in Hinweisfenster an gewhlter Position anzeigen *)
procedure THinweis.ShowString (S            : string;
                               W            : integer;
                               SAlign       : TAlignment;
                               ProVal       : integer;
                               Desc,Suffix,
                               Titel,Prompt : String;
                               WinPos       : TWinPos);
(* S      : anzuzeigender String
   W      : Breite fr S (0 = anpassen)
   SAlign : Ausrichtung des Strings
   ProVal : Wert Fortschrittsanzeige (0..100%) (<0 : aus)
   Desc   : dem String vorangestellter Text
   Suffix : dem String nachfolgender Text
   Titel  : Fensterberschrift
   Prompt : Hinweis
   WinPos : Position wie in TWinPos (s.o.) *)
var
  Org : TPoint;
begin
  InitForm (S,W,SAlign,ProVal,Desc,Suffix,Titel,Prompt);
  if not visible then begin
    Org:=GetWindowPos (Screen,Width,Height,WinPos);
    with Org do begin
      Left:=X; Top:=Y;
      end;
    Position:=poDesigned;
    Show;
    end
  else Update;
  end;

{ ---------------------------------------------------------------- }
(* Fortschrittsanzeige aktualisieren *)
procedure THinweis.ShowProgress(ProVal : integer);
begin
  Balken.Position:=ProVal;
  Update;
  end;

{ ---------------------------------------------------------------- }
(* Integerzahl in Hinweisfenster bei X,Y anzeigen *)
procedure THinweis.ShowIntegerPos (N            : longint;
                                   Nl           : word;
                                   W            : integer;
                                   Desc,Einh,
                                   Titel,Prompt : String;
                                   X,Y          : integer);
(* N      : anzuzeigende Zahl
   Nl     : Anzahl der Stellen von N
   Desc   : der Zahl vorangestellter Text
   Einh   : der Zahl nachfolgender Text (z.B. Einheiten)
   sonst wie oben *)
begin
  ShowStringPos (StrInt(N,Nl),W,taRightJustify,-1,Desc,Einh,
                 Titel,Prompt,X,Y);
  end;

{ ---------------------------------------------------------------- }
(* Integerzahl in Hinweisfenster an gewhlter Position anzeigen *)
procedure THinweis.ShowInteger (N            : longint;
                                Nl           : word;
                                W            : integer;
                                Desc,Einh,
                                Titel,Prompt : String;
                                WinPos       : TWinPos);
(* N      : anzuzeigende Zahl
   Nl     : Anzahl der Stellen von N
   Desc   : der Zahl vorangestellter Text
   Einh   : der Zahl nachfolgender Text (z.B. Einheiten)
   sonst wie oben *)
begin
  ShowString(StrInt(N,Nl),W,taRightJustify,-1,Desc,Einh,
             Titel,Prompt,WinPos);
  end;

end.

