(* Delphi Dialog
   Zahleneingabefenster mit Spin-Tasten und dyn. Inkrement
   =======================================================
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
      
   Vers. 1 - Mrz. 2002 
   *)

unit NumDlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, SysUtils, Dialogs, JrUtils, Spin;

type
  TSwitchMode = (tsNotVis,tsOff,tsOn);
  TIncrMode = (imFixed,imDecAuto,imBinAuto);

  TInputNumber = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Rahmen: TBevel;
    Descriptor1: TLabel;
    Descriptor2: TLabel;
    NumFeld1: TSpinEdit;
    NumFeld2: TSpinEdit;
    procedure NumFeld2Change(Sender: TObject);
    procedure NumFeld1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    FIncMode1,FIncMode2 : TIncrMode;
  public
    { Public declarations }
    function Execute (APos : TPoint;
                      Titel,Desc1,Desc2 : string;
                      var AVal,BVal     : longint) : boolean;
  end;

(* Zahl bearbeiten, Ergebnis: "true" bei "ok" *)
function NumDialog (APos : TPoint;
                    Titel,Desc   : string;
                    Min,Max,Incr : longint;
                    IncMode      : TIncrMode;
                    var Val      : longint) : boolean;

(* Zwei Zahlen bearbeiten, Ergebnis: "true" bei "ok" *)
function DNumDialog (APos : TPoint;
                     Titel,Desc1,Desc2   : string;
                     Min1,Max1,Incr1     : integer;
                     IncMode1            : TIncrMode;
                     Min2,Max2,Incr2     : integer;
                     IncMode2            : TIncrMode;
                     var Val1,Val2       : integer) : boolean;

var
  InputNumber: TInputNumber;

implementation

{$R *.DFM}

uses GnuGetText;

procedure TInputNumber.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{ ------------------------------------------------------------------- }
(* Text-Dialog mit 1 oder 2 Feldern anzeigen *)
function TInputNumber.Execute (APos : TPoint;
                               Titel,Desc1,Desc2 : string;
                               var AVal,BVal     : longint) : boolean;

var
  ok : boolean;
begin
  if APos.X<0 then Position:=poScreenCenter
  else with APos do begin
    Position:=poDesigned;
    if X<0 then X:=Left;
    if Y<0 then Y:=Top;
    CheckScreenBounds(Screen,x,y,Width,Height);
    Left:=x; Top:=y;
    end;
  Caption:=Titel;
  with NumFeld1 do begin
    Value:=AVal; AutoSelect:=true;
    end;
  if Desc2<>'' then begin
    NumFeld2.Visible:=true; Descriptor2.Visible:=true;
    Descriptor2.Caption:=Desc2;
    with NumFeld2 do begin
      Value:=BVal; AutoSelect:=true;
      end;
    end
  else begin
    NumFeld2.Visible:=false; Descriptor2.Visible:=false;
    end;
  Descriptor1.Caption:=Desc1;
  ActiveControl:=NumFeld1;
  ok:=ShowModal=mrOK;
  if ok then begin
    AVal:=NumFeld1.Value;
    if Desc2<>'' then BVal:=NumFeld2.Value;
    end;
  Result:=ok;
end;

procedure TInputNumber.NumFeld1Change(Sender: TObject);
begin
  with NumFeld1 do begin
    case FincMode1 of
    imDecAuto : begin
        if Value<10 then Increment:=1
        else if Value<100 then Increment:=5
        else if Value<1000 then Increment:=50
        else if Value<10000 then Increment:=500
        else Increment:=5000;
        end;
    imBinAuto: begin
        if Value<8 then Increment:=1
        else if Value<64 then Increment:=8
        else if Value<512 then Increment:=64
        else if Value<4096 then Increment:=512
        else Increment:=4096;
        end;
      end;
    end;
  end;

procedure TInputNumber.NumFeld2Change(Sender: TObject);
begin
  with NumFeld2 do begin
    case FincMode1 of
    imDecAuto : begin
        if Value<10 then Increment:=1
        else if Value<100 then Increment:=5
        else if Value<1000 then Increment:=50
        else if Value<10000 then Increment:=500
        else Increment:=5000;
        end;
    imBinAuto: begin
        if Value<8 then Increment:=1
        else if Value<64 then Increment:=8
        else if Value<512 then Increment:=64
        else if Value<4096 then Increment:=512
        else Increment:=4096;
        end;
      end;
    end;  
  end;

{ ------------------------------------------------------------------- }
(* Zahl bearbeiten, Ergebnis: "true" bei "ok" *)
function NumDialog (APos         : TPoint;
                    Titel,Desc   : string;
                    Min,Max,Incr : longint;
                    IncMode      : TIncrMode;
                    var Val      : longint) : boolean;
begin
  if not assigned(InputNumber) then InputNumber:=TInputNumber.Create(Application);
  with InputNumber do begin
    with NumFeld1 do begin
      MinValue:=Min; MaxValue:=Max; Increment:=Incr;
      end;
    FIncMode1:=IncMode;
    Result:=Execute (APos,Titel,Desc,'',Val,Val);
    Release;
    end;
  InputNumber:=nil;
  end;

{ ------------------------------------------------------------------- }
(* Zwei Zahlen bearbeiten, Ergebnis: "true" bei "ok" *)
function DNumDialog (APos                : TPoint;
                     Titel,Desc1,Desc2   : string;
                     Min1,Max1,Incr1     : integer;
                     IncMode1            : TIncrMode;
                     Min2,Max2,Incr2     : integer;
                     IncMode2            : TIncrMode;
                     var Val1,Val2       : integer) : boolean;
begin
  if not assigned(InputNumber) then InputNumber:=TInputNumber.Create(Application);
  with InputNumber do begin
    with NumFeld1 do begin
      MinValue:=Min1; MaxValue:=Max1; Increment:=Incr1;
      end;
    with NumFeld2 do begin
      MinValue:=Min2; MaxValue:=Max2; Increment:=Incr2;
      end;
    FIncMode1:=IncMode1; FIncMode2:=IncMode2;
    Result:=Execute (APos,Titel,Desc1,Desc2,Val1,Val2);
    Release;
    end;
  InputNumber:=nil;
  end;

procedure TInputNumber.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_Return then OkBtn.Click
  else if Key=VK_Cancel then CancelBtn.Click;
  end;

procedure TInputNumber.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#13 then Key:=#0;
  end;

end.
