(*  Delphi Dialog
    Zusammenstellung einer Suchpfad-Liste
    =====================================
    
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Okt. 2004
   *)
   
unit PfadDlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  SysUtils, StdCtrls, Grids;

const
  CvUp  = 1;
  CvOn  = 2;
  DefRowCount = 21;

type
  TPfadDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    PathEdit: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    DeleteBtn: TSpeedButton;
    InsertBtn: TSpeedButton;
    DirBtn: TSpeedButton;
    Liste: TListBox;
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure DirBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListeClick(Sender: TObject);
  private
    { Private declarations }
    APath : string;
  public
    { Public declarations }
    function Execute (Title,StdPath : string;
                      var AListe : string) : boolean;
  end;

var
  PfadDialog: TPfadDialog;

implementation

uses ShellDirDlg, GnuGetText;


{$R *.DFM}

procedure TPfadDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TPfadDialog.UpBtnClick(Sender: TObject);
var
  s  : string;
begin
  with Liste do if Items.Count>1 then begin
    if ItemIndex>0 then with Items do begin
      s:=Strings[ItemIndex];
      Strings[ItemIndex]:=Strings[ItemIndex-1];
      Strings[ItemIndex-1]:=s;
      ItemIndex:=ItemIndex-1;
      end;
    end;
  PathEdit.SetFocus;
  end;

procedure TPfadDialog.DownBtnClick(Sender: TObject);
var
  s  : string;
begin
  with Liste do if Items.Count>1 then begin
    if ItemIndex<Items.Count-1 then with Items do begin
      s:=Strings[ItemIndex];
      Strings[ItemIndex]:=Strings[ItemIndex+1];
      Strings[ItemIndex+1]:=s;
      ItemIndex:=ItemIndex+1;
      end;
    end;
  PathEdit.SetFocus;
  end;

procedure TPfadDialog.DeleteBtnClick(Sender: TObject);
begin
  with Liste do if ItemIndex>=0 then Items.Delete(ItemIndex);
  PathEdit.SetFocus;
  end;

procedure TPfadDialog.InsertBtnClick(Sender: TObject);
begin
  Liste.Items.Add(PathEdit.Text);
  PathEdit.SetFocus;
  end;

procedure TPfadDialog.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then InsertBtnClick(Sender);
  end;

procedure TPfadDialog.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$0D then Key:=#0;
  end;

procedure TPfadDialog.DirBtnClick(Sender: TObject);
var
   s : string;
begin
  if length(PathEdit.Text)=0 then s:=APath else s:=PathEdit.Text;
  if ShellDirDialog.Execute (Caption,false,true,APath,s) then PathEdit.Text:=s;
  end;

procedure TPfadDialog.ListeClick(Sender: TObject);
begin
  with Liste do if Items.Count>0 then PathEdit.Text:=Items[ItemIndex];
  end;

{ ------------------------------------------------------------------- }
function TPfadDialog.Execute (Title,StdPath : string;
                              var AListe : string) : boolean;
var
  ok : boolean;
  i  : integer;
  s  : string;
begin
  if Title='' then Caption:=Title;
  APath:=StdPath; s:=AListe;
  with Liste do begin
    Clear;
    PathEdit.Text:='';
    if length(s)>0 then begin
      repeat
        i:=pos(';',s);
        if i=0 then i:=succ(length(s));
        Items.Add(copy(s,1,pred(i)));
        delete (s,1,i);
        until (length(s)=0);
      PathEdit.Text:=Items[0];
      end
    else PathEdit.Text:='';
    end;
  ActiveControl:=PathEdit;
  ok:=ShowModal=mrOK;
  if ok then begin
    s:='';
    with Liste.Items do for i:=0 to Count-1 do s:=s+Strings[i]+';';
    delete (s,length(s),1);
    AListe:=s;
    end;
  Result:=ok;
  end;

end.
