(* Delphi Dialog
   Auswahl einer Farbe
   ===================
   16 Standardfarbe und 16 Benutzerfarben sind fest voreingestellt.
   Andere Farben werden ber den Stadardfarbdialog ausgewhlt.
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - June 2005
    *)
    
unit SelectColorDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, 
  Buttons, ExtCtrls, Dialogs;

type
  TColorList = array [0..15] of TColor;

  TSelectColorDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    pnlColor: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ColorDialog: TColorDialog;
    cbNoColor: TCheckBox;
    btnSelectColor: TBitBtn;
    Label3: TLabel;
    procedure pnlColorClick(Sender: TObject);
    procedure cbNoColorClick(Sender: TObject);
    procedure btnSelectColorClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    StdColPanels,SpecColPanels : array [0..15] of TPanel;
    FCol : TColor;
    procedure ShowColor;
  public
    { Public declarations }
    function Execute (ATitle   : string;
                      AList    : TColorList;
                      var ACol : TColor) : boolean;
  end;

function ColorList (c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16 : TColor) : TColorList;

var
  SelectColorDialog: TSelectColorDialog;

implementation

{$R *.DFM}

uses GnuGetText;

const
  StandardColor : TColorList = (clBlack,clMaroon,clGreen,clNavy,clOlive,
                                clPurple,clTeal,clGray,clRed,clLime,
                                clYellow,clBlue,clFuchsia,clAqua,clLtGray,
                                clWhite);

procedure TSelectColorDialog.FormCreate(Sender: TObject);
var
  i : integer;
begin
  TranslateComponent (self,'dialogs');
  for i:=0 to 15 do begin
    StdColPanels[i]:=TPanel.Create(self);
    with StdColPanels[i] do begin
      Parent:=SelectColorDialog;
      Width:=31; Height:=21;
      Left:=10+(i mod 8)*35; Top:=30+(i div 8)*25;
      BevelInner:=bvLowered; BevelOuter:=bvLowered;
      Color:=StandardColor[i];
      ParentBackground:=false;
      Tag:=i;
      OnClick:=pnlColorClick;
      end;
    end;
  for i:=0 to 15 do begin
    SpecColPanels[i]:=TPanel.Create(self);
    with SpecColPanels[i] do begin
      Parent:=SelectColorDialog;
      Width:=31; Height:=21;
      Left:=10+(i mod 8)*35; Top:=110+(i div 8)*25;
      BevelInner:=bvLowered; BevelOuter:=bvLowered;
      Color:=StandardColor[i];
      ParentBackground:=false;
      Tag:=16+i;
      OnClick:=pnlColorClick;
      end;
    end;
  pnlColor.ParentBackground:=false;
  end;

procedure TSelectColorDialog.FormDestroy(Sender: TObject);
var
  i : integer;
begin
  for i:=0 to 15 do StdColPanels[i].Free;
  for i:=0 to 15 do SpecColPanels[i].Free;
  end;

procedure TSelectColorDialog.ShowColor;
begin
  if cbNoColor.Checked then pnlColor.Color:=clBtnFace
  else pnlColor.Color:=FCol;
  end;

procedure TSelectColorDialog.pnlColorClick(Sender: TObject);
begin
  cbNoColor.Checked:=false;
  with Sender as TPanel do begin
    if Tag>15 then FCol:=SpecColPanels[Tag-16].Color
    else FCol:=StdColPanels[Tag].Color
    end;
  ShowColor;
  end;

procedure TSelectColorDialog.cbNoColorClick(Sender: TObject);
begin
  ShowColor;
  end;

procedure TSelectColorDialog.btnSelectColorClick(Sender: TObject);
begin
  with ColorDialog do begin
    Color:=FCol;
    if Execute then begin
      FCol:=Color;
      ShowColor;
      end;
    end;
  end;

{------------------------------------------------------------------- }
function TSelectColorDialog.Execute (ATitle   : string;
                                     AList    : TColorList;
                                     var ACol : TColor) : boolean;
var
  i : integer;
begin
  if length(ATitle)>0 then Caption:=ATitle;
  for i:=0 to 15 do SpecColPanels[i].Color:=AList[i];
  cbNoColor.Checked:=ACol=clNone;
  FCol:=ACol;
  ShowColor;
  if ShowModal=mrOK then begin
    if cbNoColor.Checked then Acol:=clNone
    else ACol:=FCol;
    Result:=true;
    end
  else Result:=false;
  end;

{------------------------------------------------------------------- }
function ColorList (c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16 : TColor) : TColorList;
begin
  Result[0]:=c1; Result[1]:=c2; Result[2]:=c3; Result[3]:=c4;
  Result[4]:=c5; Result[5]:=c6; Result[6]:=c7; Result[7]:=c8;
  Result[8]:=c9; Result[9]:=c10; Result[10]:=c11; Result[11]:=c12;
  Result[12]:=c13; Result[13]:=c14; Result[14]:=c15; Result[15]:=c16;
  end;

end.
