(* Delphi Dialog
   Verzeichniswahldialog mit ShellTreeView
   =======================================
   Design hnlich wie Datei-Dialog unter Windows 2000/XP

    2007 J. Rathlev 24222 Schwentinental
      Web:  www.rathlev-home.de
      Mail: info(a)rathlev-home.de

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - August 2003
   Vers. 1.1 - Sep. 2005 : changes for use with XpManifest (TPanel is transp.)
   Vers. 2.0 - Jun. 2006 : optional file window
   Vers. 2.1 - Jan. 2007 : starts on MyFiles if no directory specified
   Vers. 2.2 - Apr. 2008 : changes in ShellCtrls - see
                           http://www.kutinsoft.com/Hints/DelphiHints.php
   Vers. 2.3 - Sep. 2009 : history list for selected directories
   Vers. 2.4 - Mar. 2010 : adjustable window sizes
   *)

unit ShellDirDlg;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, ComCtrls, ShellCtrls, Menus;

const
  NetLink = 'target.lnk';

type
  TShellDirDialog = class(TForm)
    btbOK: TBitBtn;
    btbCancel: TBitBtn;
    ShellTreeView: TShellTreeView;
    spbNew: TSpeedButton;
    cbxSelectedDir: TComboBox;
    Label1: TLabel;
    spbUp: TSpeedButton;
    spbHome: TSpeedButton;
    panRoot: TPanel;
    spbNetwork: TSpeedButton;
    spbComputer: TSpeedButton;
    spbDesktop: TSpeedButton;
    spbMyFiles: TSpeedButton;
    cbxFiles: TCheckBox;
    ShellListView: TShellListView;
    PanelLeft: TPanel;
    PanelRight: TPanel;
    Panel1: TPanel;
    PopupMenu: TPopupMenu;
    itmDelete: TMenuItem;
    N1: TMenuItem;
    cancel1: TMenuItem;
    itmCreate: TMenuItem;
    itmUpdate: TMenuItem;
    Splitter: TSplitter;
    procedure ShellTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure spbDesktopClick(Sender: TObject);
    procedure spbMyFilesClick(Sender: TObject);
    procedure spbComputerClick(Sender: TObject);
    procedure spbNetworkClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure spbUpClick(Sender: TObject);
    procedure spbHomeClick(Sender: TObject);
    procedure spbNewClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ShellTreeViewClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbxFilesClick(Sender: TObject);
    procedure itmDeleteClick(Sender: TObject);
    procedure itmUpdateClick(Sender: TObject);
  private
    { Private declarations }
    FDefaultDir,FIniName,FIniSection : string;
    procedure ShowFiles (AShow : boolean);
  public
    { Public declarations }
    procedure LoadFromIni(IniName, Section : string);
    procedure ResetPosition;
    function Execute (ATitle  : string;
                      Hidden,FileView : boolean;
                      HomeDir : string;
                      var Dir : string) : boolean;
  end;

var
  ShellDirDialog: TShellDirDialog;

implementation

{$R *.dfm}

uses IniFiles, Dialogs, StrUtils, ShlObj, Shellapi, ActiveX, WinShell,
     JrUtils, FileUtils, IniFileUtils, GnuGetText;

const
  FMaxLen = 15;

{ ------------------------------------------------------------------- }
procedure TShellDirDialog.FormCreate(Sender: TObject);
var
  f : double;
begin
  TranslateComponent (self,'dialogs');
  FIniName:='';
  FIniSection:='';
  FDefaultDir:='';
  panRoot.ParentBackground:=false;
  if Screen.PixelsPerInch<>96 then begin
    PixelsPerInch:=Screen.PixelsPerInch;
    f:=PixelsPerInch/96;
    ClientWidth:=round(f*ClientWidth)+34;
    ClientHeight:=round(f*ClientHeight)+34;
    ShellListView.SetColWidths([25,13,13,18]);
    end
  else ShellListView.SetColWidths([20,10,10,14]);
  end;

const
  iniHistory = 'History';
  iniFileView = 'Fileview';
  iniTop = 'Top';
  iniLeft = 'Left';
  iniHeight = 'Height';
  iniLWidth = 'DirWidth';
  iniRWidth= 'FileWidth';

(* load history list *)
procedure TShellDirDialog.LoadFromIni(IniName, Section : string);
var
  i       : integer;
  IniFile : TIniFile;
  s       : string;
begin
  FIniName:=IniName; FIniSection:=Section;
  cbxSelectedDir.Items.Clear;
  if (length(FIniName)>0) and (length(FIniSection)>0) then begin
    IniFile:=TIniFile.Create(IniName);
    try
      for i:=0 to FMaxLen-1 do begin
        s:=IniFile.ReadString(FIniSection,iniHistory+IntToStr(i),'');
        if s<>'' then cbxSelectedDir.AddItem(s,nil);
        end;
      with IniFile do begin
        Top:=ReadInteger(FIniSection,iniTop,Top);
        Left:=ReadInteger(FIniSection,iniLeft,Left);
        Height:=ReadInteger(FIniSection,iniHeight,Height);
      with PanelLeft do Width:=ReadInteger(FIniSection,iniLWidth,Width);
      with PanelRight do Width:=ReadInteger(FIniSection,iniRWidth,Width);
      Splitter.Left:=PanelRight.Left;
        cbxFiles.Checked:=ReadBool(FIniSection,iniFileView,false);
        end;
    finally
      IniFile.Free;
      end;
    end;
  end;

procedure TShellDirDialog.ResetPosition;
begin
  Top:=50; Left:=50;
  end;

(* save history list *)
procedure TShellDirDialog.FormDestroy(Sender: TObject);
var
  i       : integer;
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then begin
    EraseSectionOfIniFile(FIniName,FIniSection);
    with cbxSelectedDir.Items do for i:=0 to Count-1 do
      WriteStringToIniFile(FIniName,FIniSection,iniHistory+IntToStr(i),Strings[i]);
    WriteIntegerToIniFile(FIniName,FIniSection,iniTop,Top);
    WriteIntegerToIniFile(FIniName,FIniSection,iniLeft,Left);
    WriteIntegerToIniFile(FIniName,FIniSection,iniHeight,Height);
    WriteIntegerToIniFile(FIniName,FIniSection,iniLWidth,PanelLeft.Width);
    WriteIntegerToIniFile(FIniName,FIniSection,iniRWidth,PanelRight.Width);
    WriteBoolToIniFile(FIniName,FIniSection,iniFileView,cbxFiles.Checked);
    UpdateIniFile(FIniName);
{    IniFile:=TIniFile.Create(FIniName);
    try
      IniFile.EraseSection (FIniSection);
      with cbxSelectedDir.Items do for i:=0 to Count-1 do
        IniFile.WriteString(FIniSection,iniHistory+IntToStr(i),Strings[i]);
      with IniFile do begin
        WriteInteger(FIniSection,iniTop,Top);
        WriteInteger(FIniSection,iniLeft,Left);
        WriteBool(FIniSection,iniFileView,cbxFiles.Checked);
        end;
    finally
      IniFile.Free;
      end;    }
    end;
  end;

{ ------------------------------------------------------------------- }
(* Initialize *)
procedure TShellDirDialog.FormShow(Sender: TObject);
begin
  FitToScreen(Screen,self);
  with spbHome do begin
    Visible:=FDefaultDir<>'';
    Hint:=dgettext('dialogs','Default: ')+FDefaultDir;
    end;
  with cbxSelectedDir do begin
    if Items.Count>0 then Style:=csDropDown else Style:=csSimple;
    end;
  end;


procedure TShellDirDialog.FormActivate(Sender: TObject);
begin
  with ShellTreeView do begin
//      Path:=cbxSelectedDir.Text;
    try
      Selected.MakeVisible;
    except
      end;
    SetFocus;
    end;
  end;

{------------------------------------------------------------------- }
(* go to parent directory *)
procedure TShellDirDialog.spbUpClick(Sender: TObject);
begin
  with ShellTreeView do begin
//    s:=SelectedFolder.Parent.PathName;
    if (Selected.Parent=nil) then begin
      Root:='rfMyComputer'; // Path:='';
      end
    else if (Selected.Parent.Level>0) then Path:=SelectedFolder.Parent.PathName;
    Selected.MakeVisible;
    SetFocus;
    end;
{  s:=cbxSelectedDir.Text;
  if (copy(s,1,2)<>'\\') or (PosEx('\',s,3)>0) then begin
    while s[length(s)]<>'\' do delete (s,length(s),1);
    delete (s,length(s),1);
    if length(s)>0 then begin
      if (length(s)=2) and (copy(s,2,1)=':') then begin
        ShellTreeView.Root:='rfMyComputer'; s:=s+'\';
        end;
      with ShellTreeView do begin
        Path:=s; Selected.MakeVisible;
        end;
      end;
    end;
  ShellTreeView.SetFocus;   }
  end;

(* go to home directory *)
procedure TShellDirDialog.spbHomeClick(Sender: TObject);
begin
  if not SetCurrentDir(FDefaultDir) then begin
    ErrorDialog(Format(dgettext('dialogs','Directory not found:'+sLineBreak+'%s!'),[FDefaultDir]));
    with cbxSelectedDir do begin
      RemoveFromHistory(Items,FDefaultDir);
      ItemIndex:=0;
      end;
    end
  else with ShellTreeView do begin
    if (Root='rfNetwork') or ((length(FDefaultDir)=3) and (copy(FDefaultDir,2,2)=':\')) then Root:=FDefaultDir
    else begin
      Root:='rfMyComputer';
      try Path:=FDefaultDir; except end;
      Selected.MakeVisible;
      end;
    end;
  ShellTreeView.SetFocus;
  end;

(* create new directory *)
procedure TShellDirDialog.spbNewClick(Sender: TObject);
var
  s : string;
begin
  s:='';
  if InputQuery (ShellTreeView.Path,dgettext('dialogs','New subdirectory:'),s) then begin
    s:=IncludeTrailingPathDelimiter(ShellTreeView.Path)+s;
    if not ForceDirectories(s) then
      ErrorDialog(Format(dgettext('dialogs','Could not create directory:'+sLineBreak+'%s!'),[s]))
    else with ShellTreeView do begin
      Root:='rfMyComputer';
      try Path:=s; except end;
      Selected.MakeVisible;
      end;
    end;
  ShellTreeView.SetFocus;
  end;

procedure TShellDirDialog.itmDeleteClick(Sender: TObject);
var
  s : string;
begin
  s:=ShellTreeView.Path;
  if (DirFiles(s,true)>0) and
    not ConfirmDialog(dgettext('dialogs','Directory is not empty, delete anyway?')) then Exit;
  spbUpClick(Sender);
  if not DeleteDirectory(s) then
    ErrorDialog(Format(dgettext('dialogs','Error deleting directory:'+sLineBreak+'%s!'),[s]));
  end;

procedure TShellDirDialog.itmUpdateClick(Sender: TObject);
begin
  with ShellTreeView do Refresh(Selected);
  end;

{------------------------------------------------------------------- }
procedure TShellDirDialog.ShellTreeViewClick(Sender: TObject);
begin
{  with ShellTreeView do if assigned(Selected) then begin
    Refresh(Selected);
    Selected.MakeVisible;
    end;    }
  end;

procedure TShellDirDialog.ShellTreeViewChange(Sender: TObject;
  Node: TTreeNode);
var
  s,s1,s2,s3,s4 : string;
begin
  if Active then with ShellTreeView,SelectedFolder do begin
    btbOk.Enabled:=fpFileSystem in Properties;
    cbxSelectedDir.Text:=PathName;
    if fpIsLink in Properties then begin
      s:=IncludeTrailingPathDelimiter(PathName)+NetLink;
      if FileExists(s) then begin
        GetLink(s,s1,s2,s3,s4);
        cbxSelectedDir.Text:=s1;
        end
      end;
    end;
  end;

procedure TShellDirDialog.spbDesktopClick(Sender: TObject);
begin
  ShellTreeView.Root:='rfDesktop';
  end;

procedure TShellDirDialog.spbMyFilesClick(Sender: TObject);
begin
  ShellTreeView.Root:='rfPersonal';
  end;

procedure TShellDirDialog.spbComputerClick(Sender: TObject);
begin
  ShellTreeView.Root:='rfMyComputer';
  end;

procedure TShellDirDialog.spbNetworkClick(Sender: TObject);
begin
  ShellTreeView.Root:='rfNetwork';
  end;

procedure TShellDirDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[]) and (Key=VK_F5) then
    with ShellTreeView do if assigned(Selected) then Refresh(Selected);
  end;

procedure TShellDirDialog.ShowFiles (AShow : boolean);
var
  d : integer;
begin
  if AShow then with PanelRight do begin
//    Width:=301;
    d:=Left+Width;
    Visible:=true;
    end
  else with PanelLeft do begin
    d:=Width;
    PanelRight.Visible:=false;
    end;
  ClientWidth:=d;
  end;

procedure TShellDirDialog.cbxFilesClick(Sender: TObject);
begin
  if Visible then ShowFiles(cbxFiles.Checked);
  end;

{------------------------------------------------------------------- }
(* Dialog an Position anzeigen *)
function TShellDirDialog.Execute (ATitle  : string;
                                  Hidden,FileView  : boolean;
                                  HomeDir : string;
                                  var Dir : string) : boolean;
var
  ok : boolean;
  s,r : string;
begin
  Caption:=ATitle; FDefaultDir:=HomeDir;
  s:=Dir;
  if length(s)=0 then s:=HomeDir;
  if (length(s)=0) or not DirectoryExists(s)then begin
    s:=GetDesktopFolder(CSIDL_PERSONAL);
    r:='rfPersonal';
    if length(s)=0 then begin
      s:=GetCurrentDir;
      r:='rfMyComputer';
      end;
    end
  else begin
    if copy(s,1,2)='\\' then r:='rfNetwork' else r:='rfMyComputer';
    end;
  cbxSelectedDir.Text:=s;
  with ShellTreeView do begin
    Root:=r;
    if Hidden then ObjectTypes:=ObjectTypes+[otHidden]
    else ObjectTypes:=ObjectTypes-[otHidden];
    Path:=s;
    if assigned(Selected) then try Selected.Expand(false); except end;
    end;
  cbxFiles.Visible:=FileView;
  if FileView then ShowFiles(cbxFiles.Checked)
  else ShowFiles(false);
  ok:=ShowModal=mrOK;
  if ok then with cbxSelectedDir do begin
    Dir:=Text;
    AddToHistory(Items,Text);
    end;
  Result:=ok;
  end;

end.
