(* Delphi Dialog
   Anzeige einer Log-Datei (z.B. von Personal Backup)
   ==================================================
   Textanzeige mit TRichEdit
   Suchfunktionen fr 2 spez. Textteile
   Drucken (auch von markierten Textteilen)
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Sep 2004
   *)


unit ShowRtf;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Printers, Search, JrUtils, ComCtrls, PrintPageDlg,
  RichEdit, Math;

const
  defTopMargin = 150;  // Angaben in 1/10 mm
  defBottomMargin= 150;
  defLeftMargin = 200;
  defRightMargin= 100;
  defFontName = 'Courier New';
  defFontSize = 10;
  defOrientation = poPortrait;

type
  TShowDlgBtn = (sbOpen,sbErase,sbPrint,sbSearch,sbSection,sbError);
  TShowDlgButtons = set of TShowDlgBtn;

  TShowDlgType = (stShow,stModal);

  TShowtextDialog = class(TForm)
    Panel2: TPanel;
    EndeBtn: TBitBtn;
    PrintBtn: TBitBtn;
    PrintDialog: TPrintDialog;
    DeleteBtn: TBitBtn;
    RichEdit: TRichEdit;
    FindDialog: TFindDialog;
    SearchBtn: TBitBtn;
    NextSectBtn: TBitBtn;
    PrevErrBtn: TBitBtn;
    PrevSectBtn: TBitBtn;
    NextErrBtn: TBitBtn;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    OpenBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RichEditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PrintBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure RichEditKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure RichEditChange(Sender: TObject);
    procedure RichEditMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FindDialogFind(Sender: TObject);
    procedure SearchBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NextSectBtnClick(Sender: TObject);
    procedure PrevErrBtnClick(Sender: TObject);
    procedure PrevSectBtnClick(Sender: TObject);
    procedure NextErrBtnClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    LPos             : integer;
    FIniName,FIniSection,
    FTitle,FName,FFilter,
    SectText,ErrText : string;
    procedure Print (NoPrompt,SelectedLines : boolean);
    function LoadText (const Filename : string) : boolean;
  public
    { Public-Deklarationen }
    PrinterSettings : TPrinterSettings;
    procedure ReadFromIni (AIniName,AIniSection : string);
    procedure PrintFile (TextDatei: string);
    procedure Execute (const Title,TextDatei,
                       PrevCap1,NextCap1,SrchText1,
                       PrevCap2,NextCap2,SrchText2,
                       Filter                   : string;
                       Position  : TPoint;
                       Line      : integer;
                       DlgType   : TShowDlgType;
                       Buttons   : TShowDlgButtons);
  end;

var
  ShowtextDialog: TShowtextDialog;

implementation

{$R *.DFM}

uses GnuGetText, IniFiles;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.FormCreate(Sender: TObject);
begin
  // In TranslateComponent wird versucht, eine leere Zeile in TRichEdit einzufgen.
  // Das verursacht einen Fehler bei Riched20.dll ab Vers. 3.50
  // siehe sRichEditInsertError ind ComCtrls.TRichEditStrings.Insert
  try
    TranslateComponent (self,'dialogs');
  except end;
  if Screen.PixelsPerInch<>96 then begin
    PixelsPerInch:=Screen.PixelsPerInch;
    ClientWidth:=round(PixelsPerInch/96*ClientWidth)+34;
    end;
  RichEdit.Clear;
  with PrinterSettings do begin
    with Margins do begin
      Top:=defTopMargin;
      Bottom:=defBottomMargin;
      Left:=defLeftMargin;
      Right:=defRightMargin;
      end;
    FontName:=defFontName;
    FontSize:=defFontSize;
    Orientation:=defOrientation;
    end;
  FIniName:=''; FIniSection:='';
  end;

const
  IniLeftMarg = 'LeftMargin';
  IniRightMarg = 'RightMargin';
  IniTopMarg = 'TopMargin';
  IniBottomMarg = 'BottomMargin';
  IniFontName = 'FontName';
  IniFontSize = 'FontSize';
  IniOrientation = 'Orientation';

procedure TShowtextDialog.ReadFromIni (AIniName,AIniSection : string);
var
  n : integer;
begin
  FIniName:=AIniName; FIniSection:=AIniSection;
  if (length(FIniName)>0) and (length(FiniSection)>0) then
      with TIniFile.Create(FIniName),PrinterSettings do begin
    try
      with Margins do begin
        Top:=ReadInteger(FIniSection,IniTopMarg,defTopMargin);
        Bottom:=ReadInteger(FIniSection,IniBottomMarg,defBottomMargin);
        Left:=ReadInteger(FIniSection,IniLeftMarg,defLeftMargin);
        Right:=ReadInteger(FIniSection,IniRightMarg,defRightMargin,);
        end;
      FontName:=ReadString(FIniSection,IniFontName,defFontName);
      FontSize:=ReadInteger(FIniSection,IniFontSize,defFontSize);
      n:=ReadInteger (FIniSection,IniOrientation,ord(Orientation));
      if (n<0) or (n>1) then n:=ord(Orientation);
      Orientation:=TPrinterOrientation(n);
    finally
      Free;
      end;
    end;
  end;

procedure TShowtextDialog.FormDestroy(Sender: TObject);
begin
  if (length(FIniName)>0) and (length(FiniSection)>0) then
      with TIniFile.Create(FIniName),PrinterSettings do begin
    try
      with Margins do begin
        WriteInteger(FIniSection,IniTopMarg,Top);
        WriteInteger(FIniSection,IniBottomMarg,Bottom);
        WriteInteger(FIniSection,IniLeftMarg,Left);
        WriteInteger(FIniSection,IniRightMarg,Right);
        end;
      WriteString(FIniSection,IniFontName,FontName);
      WriteInteger(FIniSection,IniFontSize,FontSize);
      WriteInteger (FIniSection,IniOrientation,ord(Orientation));
    finally
      Free;
      end;
    end;
  end;

{ ---------------------------------------------------------------- }
(* Cursorposition anzeigen *)
procedure TShowtextDialog.RichEditChange(Sender: TObject);
var
  pt : TPoint;
begin
  pt:=Richedit.CaretPos;
  StatusBar.Panels[0].Text:=Format(dgettext('dialogs',' Line: %u of %u'),[pt.y+1,Richedit.Lines.Count+1]);
  end;

procedure TShowtextDialog.FormShow(Sender: TObject);
var
  LineNr : integer;
begin
  with Richedit do begin
    if (LPos<=0) or (LPos>=Lines.Count) then LineNr:=Lines.Count else LineNr:=LPos-1;
    SelStart:=Perform(EM_LINEINDEX,LineNr,0);
    Perform(EM_SCROLLCARET,0,0);
    if LineNr=Lines.Count then SelLength:=0
    else SelLength:=Perform(EM_LINEINDEX,LineNr+1,0)-SelStart;
    StatusBar.Panels[0].Text:=Format(dgettext('dialogs',' Line: %u of %u'),[LineNr+1,Richedit.Lines.Count+1]);
    end;
  with FindDialog do Options:=Options -[frDown];
  end;

procedure TShowtextDialog.FormPaint(Sender: TObject);
begin
  RicheditChange(Sender);
  end;

procedure TShowtextDialog.RichEditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_ESCAPE then ModalResult:=mrCancel;
  end;

procedure TShowtextDialog.RichEditKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[ssCtrl]) and (Key=ord('F')) then SearchBtnClick(Sender);
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  if (Key=VK_DELETE)  and (RichEdit.SelLength>0) then begin
    if ConfirmDialog (Caption,dgettext('dialogs','Delete selected text?'),
          Point(Left+200,Top+100)) then begin
      with RichEdit do begin
        ReadOnly:=false;
        ClearSelection;
        Lines.SaveToFile(FName);
        ReadOnly:=true;
        end;
      end;
    end;
  RicheditChange(Sender);
  end;

procedure TShowtextDialog.FormActivate(Sender: TObject);
begin
  Richedit.SetFocus;
  end;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.Print (NoPrompt,SelectedLines : boolean);
var
  w,v       : TSize;
  i,k,lh,
  n1,n2,
  y,lp,hp   : integer;
  txt,s,t   : string;
  Selection : TCharRange;
  f         : double;

const
  PtToCm = 0.0353;   (* 1 Pt in cm *)

  procedure CheckForNewPage;
  begin
    with Printer,Canvas do begin
      if y>=lp then begin
        y:=PrinterSettings.Margins.Top;
        newpage;
        end;
      if y=PrinterSettings.Margins.Top then begin   // Kopfzeile erzeugen
        font.style := [fsbold];
        txt:=Format(dgettext('dialogs','Page: %u'),[pagenumber]);
        textout(PrinterSettings.Margins.Left,y,StripPath(Caption,72));
        textout(hp-TextWidth(txt),y,txt);
        inc(y,lh+3);
        moveto (PrinterSettings.Margins.Left,y); lineto (hp,y);
        font.style := [];
        inc(y,20);
        end;
      end;
    end;

begin
  if NoPrompt or PrintDialog.Execute then begin
    Screen.Cursor:=crHourglass;
    with Printer do begin
      Orientation:=PrinterSettings.Orientation;
      lp:=GetDeviceCaps (Handle,VERTSIZE);
      if lp<=0 then lp:=2900 else lp:=10*lp;  // Seitenhhe in 1/10 mm
      hp:=GetDeviceCaps (Handle,HORZSIZE);
      if hp<=0 then hp:=2000 else hp:=10*hp;  // Seitenbreite in 1/10 mm
      Title:=Caption;
      Begindoc;
      with Canvas do begin
        setmapmode(handle,mm_lometric);     // 1/10 mm aber Y geht nach oben
        GetWindowExtEx(handle,w);           // Auflsung ermitteln
        GetViewportExtEx(handle,v);
        setmapmode(handle,MM_ANISOTROPIC);         // neuer Abb.-Modus
        SetWindowExtEX(handle,w.cx,w.cy,nil);      // Skalierung ...
        SetViewPortExtEx(handle,v.cx,-v.cy,nil);
        f:=PtToCm*100*PrinterSettings.FontSize;    (* Punkte in 0,1 mm umrechnen *)
        lh:=round(f);
        with Font do begin
          Name:=PrinterSettings.FontName;
//          Size:=PrinterSettings.FontSize;
          Height:=lh;
          end;
        Brush.Color := clwhite;   // Hintergrundfarbe
        lp:=lp-PrinterSettings.Margins.Bottom;
        hp:=hp-PrinterSettings.Margins.Right;
        y:=PrinterSettings.Margins.Top;

        with RichEdit do begin
          if SelectedLines then begin
            (* Zeilennummern des selektierten Bereichs ermitteln *)
            Perform(EM_EXGETSEL,0,longint(@Selection));
            with Selection do begin
              n1:=Perform(EM_LINEFROMCHAR,cpMin,0);
              n2:=Perform(EM_LINEFROMCHAR,cpMax,0);
              end;
            end
          else begin
            n1:=0; n2:=Lines.Count-1;
            end;
          end;
        for i:=n1 to n2 do begin
          CheckForNewPage;
          txt:=RichEdit.Lines[i];
          while TextWidth(txt)>=hp-PrinterSettings.Margins.Left do begin
            s:='';
            repeat
              k:=Min(Pos('\',txt),Pos(' ',txt));
              if k>0 then begin
                t:=copy(txt,1,k);
                if TextWidth(s+t)<hp-PrinterSettings.Margins.Left then begin
                  s:=s+t; delete(txt,1,k); t:='';
                  end;
                end
              else t:='x';
              until length(t)>0;
            textout(PrinterSettings.Margins.Left,y,s);
            txt:='* '+txt;
            inc(y,lh+3);
            CheckForNewPage;
            end;
          textout(PrinterSettings.Margins.Left,y,txt);
          inc(y,lh+3);
          end;
        end;
      EndDoc;
      end;
    Screen.Cursor:=crDefault;
    end;
  end;

procedure TShowtextDialog.PrintBtnClick(Sender: TObject);
var
  sel      : boolean;
begin
  if not Printer.Printing then begin
    if RichEdit.SelLength>0 then sel:=ConfirmDialog('',dgettext('dialogs','Print selected lines?'))
    else sel:=false;
    Print(false,sel);
    Richedit.SetFocus;
    end;
  end;

{ ------------------------------------------------------------------- }
function TShowtextDialog.LoadText (const Filename : string) : boolean;
var
  fs : TStream;
begin
  FName:=Filename;
  Result:=false;
  with RichEdit do begin
    fs:= TFileStream.Create(FName,fmOpenRead or fmShareDenyNone);
    try
      Lines.LoadFromStream(fs);
      Result:=true;
    finally
      fs.Free;
      end;
    end;
  end;

procedure TShowtextDialog.OpenBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(FName)>0 then InitialDir:=ExtractFilePath(FName)
    else InitialDir:='';
    Title:=FTitle;
    Filter:=FFilter;
    Filename:='';
    if Execute then begin
      LoadText(Filename);
      Caption:=Title+' ['+FName+']';
      end;
    end;
  end;

procedure TShowtextDialog.DeleteBtnClick(Sender: TObject);
begin
  if RichEdit.SelLength>0 then begin
    if ConfirmDialog (Caption,dgettext('dialogs','Delete selected text?'),
          Point(Left+200,Top+100)) then begin
      with RichEdit do begin
        ReadOnly:=false;
        ClearSelection;
        Lines.SaveToFile(FName);
        ReadOnly:=true;
        end;
      end;
    end
  else begin
    if ConfirmDialog (Caption,Format(dgettext('dialogs','Delete file "%s"?'),[FName]),
          Point(Left+200,Top+100)) then begin
      DeleteFile(FName);
      RichEdit.Clear;
      end;
    ModalResult:=mrCancel;
    end;
  end;

procedure TShowtextDialog.RichEditMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  RichEditChange(Sender);
  end;

procedure TShowtextDialog.FindDialogFind(Sender: TObject);
begin
  with FindDialog do
    if not SearchMemo(Richedit,FindText,Options) then
      ShowMessage(Format(dgettext('dialogs','"%s" not found!'),[FindText]))
    else RicheditChange(Sender);
  end;

procedure TShowtextDialog.SearchBtnClick(Sender: TObject);
begin
  FindDialog.Execute;
  end;

procedure TShowtextDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssCtrl in Shift) and (Key=ord('F')) then FindDialog.Execute;
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  end;

procedure TShowtextDialog.PrevSectBtnClick(Sender: TObject);
begin
  with FindDialog do begin
    FindText:=SectText; Options:=Options-[frDown];
    if not SearchMemo(Richedit,FindText,Options) then begin
      Richedit.SelStart:=0;
      StatusBar.Panels[1].Text:=dgettext('dialogs','No more sections!');
      end
    else StatusBar.Panels[1].Text:='';
    RicheditChange(Sender);
    end;
  end;

procedure TShowtextDialog.NextSectBtnClick(Sender: TObject);
var
  pt : TPoint;
begin
  with FindDialog do begin
    FindText:=SectText; Options:=Options+[frDown];
    if not SearchMemo(Richedit,FindText,Options) then with Richedit do begin
      pt:=CaretPos; pt.y:=Lines.Count; CaretPos:=pt;
      StatusBar.Panels[1].Text:=dgettext('dialogs','No more sections!');
      end
    else StatusBar.Panels[1].Text:='';
    RicheditChange(Sender);
    end;
  end;

procedure TShowtextDialog.PrevErrBtnClick(Sender: TObject);
begin
  with FindDialog do begin
    FindText:=ErrText; Options:=Options-[frDown];
    if not SearchMemo(Richedit,FindText,Options) then begin
      Richedit.SelStart:=0;
      StatusBar.Panels[1].Text:=dgettext('dialogs','No more errors!');
      end
    else StatusBar.Panels[1].Text:='';
    RicheditChange(Sender);
    end;
  end;

procedure TShowtextDialog.NextErrBtnClick(Sender: TObject);
var
  pt : TPoint;
begin
  with FindDialog do begin
    FindText:=ErrText; Options:=Options+[frDown];
    if not SearchMemo(Richedit,FindText,Options) then with Richedit do begin
      pt:=CaretPos; pt.y:=Lines.Count; CaretPos:=pt;
      StatusBar.Panels[1].Text:=dgettext('dialogs','No more errors!');
      end
    else StatusBar.Panels[1].Text:='';
    RicheditChange(Sender);
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.PrintFile (TextDatei : string);
begin
  if FileExists (TextDatei) then begin
    RichEdit.Lines.LoadFromFile(TextDatei);
    Caption:=ExtractFileName(TextDatei);
    Print(true,false);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Text anzeigen *)
procedure TShowtextDialog.Execute (const Title,TextDatei,
                                   PrevCap1,NextCap1,SrchText1,
                                   PrevCap2,NextCap2,SrchText2,
                                   Filter                   : string;
                                   Position  : TPoint;
                                   Line      : integer;
                                   DlgType   : TShowDlgType;
                                   Buttons   : TShowDlgButtons);
var
  x  : integer;
begin
  if Visible then BringToFront
  else begin
    with Position do begin
      Left:=x; Top:=y;
      end;
    FTitle:=Title;
    Caption:=Title+' ['+TextDatei+']';
    FFilter:=Filter;
    LPos:=Line;
    x:=2;
    with OpenBtn do if sbOpen in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with PrintBtn do if sbPrint in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      Enabled:=Printer.Printers.Count>0;
      end
    else Visible:=false;
    with DeleteBtn do if sbErase in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with SearchBtn do if sbSearch in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    if sbSection in Buttons then begin
      SectText:=SrchText1;
      with PrevSectBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=PrevCap1;
        end;
      with NextSectBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=NextCap1;
        end;
      end
    else begin
      PrevSectBtn.Visible:=false;
      NextSectBtn.Visible:=false;
      end;
    if sbError in Buttons then begin
      ErrText:=SrchText2;
      with PrevErrBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=PrevCap2;
        end;
      with NextErrBtn do begin
        Visible:=true; Left:=x;
        Hint:=NextCap2;
        end;
      end
    else begin
      PrevErrBtn.Visible:=false;
      NextErrBtn.Visible:=false;
      end;
    if FileExists (TextDatei) then begin
      LoadText(TextDatei);
      if DlgType=stModal then ShowModal else Show;
      end
    else ErrorDialog ('',dgettext('dialogs','File: ')+Format(dgettext('dialogs','"%s" not found!'),[TextDatei]));
    end;
  end;

end.
