(* Delphi Dialog
   Text anzeigen (Basis: TMemo)
   =============
   mit Suchfunktion und Drucken

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Sep. 2001
   *)

unit ShowText;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Buttons, Printers, Search, JrUtils, Math, ComCtrls;

const
  texthoehe  = 40;   // 4 mm
  obererRand = 150;  // 150 mm
  linkerRand = 200;  // 20 mm
  untererRand = 150; // 15 mm
  rechterRand = 100; // 10 mm

type
  TShowDlgBtn = (sbErase,sbPrint,sbSearch);
  TShowDlgButtons = set of TShowDlgBtn;

  TShowDlgType = (stShow,stShowModal);

  TShowtextDialog = class(TForm)
    Panel2: TPanel;
    EndeBtn: TBitBtn;
    PrintBtn: TBitBtn;
    PrintDialog: TPrintDialog;
    DeleteBtn: TBitBtn;
    SearchBtn: TBitBtn;
    FindDialog: TFindDialog;
    Memo: TMemo;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PrintBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
    procedure FindDialogFind(Sender: TObject);
    procedure SearchBtnClick(Sender: TObject);
    procedure MemoChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormDeactivate(Sender: TObject);
  private
    { Private-Deklarationen }
    LPos   : integer;
    FName  : string;
  public
    { Public-Deklarationen }
    procedure Execute (TextDatei : string;
                       Position  : integer;
                       DlgType   : TShowDlgType;
                       Buttons   : TShowDlgButtons);
  end;

var
  ShowtextDialog: TShowtextDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  Memo.Clear;
  end;

{ ---------------------------------------------------------------- }
(* Cursorposition anzeigen *)
procedure TShowtextDialog.MemoChange(Sender: TObject);
var
  pt : Tpoint;
begin
  with Memo do begin
    pt:=CaretPos;
    StatusBar.SimpleText:=dgettext('dialogs',' Line: ')+IntToStr(pt.y+1)+dgettext('dialogs',' of ')+IntToStr(Lines.Count+1);
//    if LinesVisible<Lines.Count then ScrollBars:=ssVertical else ScrollBars:=ssNone;
    end;
  end;

procedure TShowtextDialog.FormShow(Sender: TObject);
var
  LineNr : integer;
begin
  with Memo do begin
    if (LPos<=0) or (LPos>=Lines.Count) then LineNr:=Lines.Count else LineNr:=LPos-1;
    SelStart:=Perform(EM_LINEINDEX,LineNr,0);
    Perform(EM_SCROLLCARET,0,0);
    if LineNr=Lines.Count then SelLength:=0
    else SelLength:=Perform(EM_LINEINDEX,LineNr+1,0)-SelStart;
//    if (LineNr>LinesVisible) then SendMessage (Handle,EM_LINESCROLL,0,TextLine-LinesVisible+4);
    end;
  MemoChange(Sender);
  end;

procedure TShowtextDialog.FormPaint(Sender: TObject);
begin
  MemoChange(Sender);
  end;

procedure TShowtextDialog.MemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_ESCAPE then ModalResult:=mrCancel;
  end;

procedure TShowtextDialog.MemoKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[ssCtrl]) and (Key=ord('F')) then SearchBtnClick(Sender);
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  MemoChange(Sender);
  end;

procedure TShowtextDialog.FormActivate(Sender: TObject);
begin
  Memo.SetFocus;
  end;

procedure TShowtextDialog.FormDeactivate(Sender: TObject);
begin
  end;

{ ------------------------------------------------------------------- }
(* Text anzeigen *)
procedure TShowtextDialog.Execute (TextDatei : string;
                                   Position  : integer;
                                   DlgType   : TShowDlgType;
                                   Buttons   : TShowDlgButtons);
var
  x : integer;
begin
  if Visible then BringToFront
  else begin
    Caption:=TextDatei;
    LPos:=Position;
    x:=2;
    with PrintBtn do if sbPrint in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with DeleteBtn do if sbErase in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with SearchBtn do if sbSearch in Buttons then begin
      Visible:=true; Left:=x;
      end
    else Visible:=false;
    with EndeBtn do if DlgType=stShowModal then Caption:=dgettext('dialogs',' &Back')
    else Caption:=dgettext('dialogs',' &Close');
    if FileExists (TextDatei) then begin
      FName:=TextDatei;
      with Memo do begin
        Lines.LoadFromFile(TextDatei);
        end;
      if DlgType=stShowModal then ShowModal else Show;
      end
    else ErrorDialog ('',dgettext('dialogs','File: ')+Format(dgettext('dialogs','"%s" not found!'),[TextDatei]));
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.PrintBtnClick(Sender: TObject);
var
  w,v      : TSize;
  i,k,n1,n2,
  y,lp,hp  : integer;
  sel      : boolean;
  txt,s,t  : string;

  procedure CheckForNewPage;
  begin
    with Printer,Canvas do begin
      if y>=lp then begin
        y:=obererRand;
        newpage;
        end;
      if y=obererRand then begin   // Kopfzeile erzeugen
        font.style := [fsbold];
        txt:=dgettext('dialogs','Page: ')+IntToStr(pagenumber);
        textout(linkerRand,y,StripPath(Caption,72));
        textout(hp-TextWidth(txt),y,txt);
        inc(y,texthoehe+3);
        moveto (linkerRand,y); lineto (hp,y);
        font.style := [];
        inc(y,20);
        end;
      end;
    end;

begin
  if not Printer.Printing then begin
    if Memo.SelLength>0 then sel:=ConfirmDialog('',dgettext('dialogs','Print selected lines?'))
    else sel:=false;
    if PrintDialog.Execute then begin
      Screen.Cursor:=crHourglass;
      with Printer do begin
        lp:=GetDeviceCaps (Handle,VERTSIZE);
        if lp<=0 then lp:=2900 else lp:=10*lp;  // Seitenhhe in 1/10 mm
        hp:=GetDeviceCaps (Handle,HORZSIZE);
        if hp<=0 then hp:=2000 else hp:=10*hp;  // Seitenbreite in 1/10 mm
        Title:=Caption;
        begindoc;
        with Canvas do begin
          setmapmode(handle,mm_lometric);     // 1/10 mm aber Y geht nach oben
          GetWindowExtEx(handle,w);           // Auflsung ermitteln
          GetViewportExtEx(handle,v);
          setmapmode(handle,MM_ANISOTROPIC);         // neuer Abb.-Modus
          SetWindowExtEX(handle,w.cx,w.cy,nil);      // Skalierung ...
          SetViewPortExtEx(handle,v.cx,-v.cy,nil);
          font.name := 'Courier New';
          Brush.Color := clwhite;   // Hintergrundfarbe
          font.height:= texthoehe;
          lp:=lp-untererRand;
          hp:=hp-rechterRand;
          y:=obererRand;
          with Memo do begin
            if sel then begin
              (* Zeilennummern des selektierten Bereichs ermitteln *)
              n1:=Perform(EM_LINEFROMCHAR,SelStart,0);
              n2:=Perform(EM_LINEFROMCHAR,SelStart+SelLength,0);
              end
            else begin
              n1:=0; n2:=Lines.Count-1;
              end;
            end;
          for i:=n1 to n2 do begin
            CheckForNewPage;
            txt:=Memo.Lines[i];
            while TextWidth(txt)>=hp-linkerRand do begin
              s:='';
              repeat
                k:=Min(Pos('\',txt),Pos(' ',txt));
                if k>0 then begin
                  t:=copy(txt,1,k);
                  if TextWidth(s+t)<hp-linkerRand then begin
                    s:=s+t; delete(txt,1,k); t:='';
                    end;
                  end
                else t:='x';
                until length(t)>0;
              textout(linkerRand,y,s);
              txt:='* '+txt;
              inc(y,texthoehe+3);
              CheckForNewPage;
              end;
            textout(linkerRand,y,txt);
            inc(y,texthoehe+3);
            end;
          end;
        EndDoc;
        end;
      Screen.Cursor:=crDefault;
      end;
    Memo.SetFocus;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShowtextDialog.DeleteBtnClick(Sender: TObject);
begin
  if ConfirmDialog ('',Format(dgettext('dialogs','Delete file "%s"?'),[FName]),Point(Left+200,Top+100)) then begin
    DeleteFile(Caption);
    Memo.Clear;
    end;
  Memo.SetFocus;  
  end;

procedure TShowtextDialog.EndeBtnClick(Sender: TObject);
begin
  Close;
  end;

procedure TShowtextDialog.FindDialogFind(Sender: TObject);
begin
  with Sender as TFindDialog do
    if not SearchMemo(Memo, FindText, Options) then
      ShowMessage(Format(dgettext('dialogs','"%s" not found!'),[FindText]));
  end;

procedure TShowtextDialog.SearchBtnClick(Sender: TObject);
begin
  FindDialog.Execute;
  end;

procedure TShowtextDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssCtrl in Shift) and (Key=ord('F')) then FindDialog.Execute;
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  end;

end.
