(* Delphi-Unit
   Hinweisfenster fr Status mit Fortschrittsanzeige (opt.)
   ========================================================
   
    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Okt. 2002 
   *)

unit StatWind;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, Prgrss;

const
  Rand = 5;
  Breite = 271;

type
  TStatusWindow = class(TForm)
    StatusLabel: TLabel;
    BarProgress: TBarProgress;
    btnCancel: TBitBtn;
    ProgLabel: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure SetStatus (Value: string);
    procedure SetProgress (Value : string);
  public
    { Public declarations }
    Stopped : boolean;
    procedure SetColor (FgCol : TColor);
    procedure Start;
    procedure Stop;
    procedure ShowStatus (Pos        : TPoint;
                          Titel,Stat : string;
                          AllowBreak : boolean;
                          AIncrement : integer);
    property Status : string write SetStatus;
    property Progress : string write SetProgress;
  end;

var
  StatusWindow: TStatusWindow;

implementation

{$R *.dfm}

uses FileCtrl, GnuGetText;

{ ---------------------------------------------------------------- }
procedure TStatusWindow.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TStatusWindow.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  BarProgress.Status:=psOff;
  end;

{ ---------------------------------------------------------------- }
procedure TStatusWindow.btnCancelClick(Sender: TObject);
begin
  Stopped:=true;
  end;

procedure TStatusWindow.SetColor (FgCol : TColor);
begin
  with BarProgress do begin
    BarColor:=FgCol;
    end;
  end;

procedure TStatusWindow.SetStatus (Value : string);
begin
  with StatusLabel do Caption:=MinimizeName(Value,Canvas,Barprogress.Width);
  end;

procedure TStatusWindow.SetProgress (Value : string);
begin
  ProgLabel.Caption:=Value;
  end;

{ ---------------------------------------------------------------- }
procedure TStatusWindow.Start;
begin
  BarProgress.Status:=psOn;
  end;

procedure TStatusWindow.Stop;
begin
  BarProgress.Status:=psOff;
  end;

{ ---------------------------------------------------------------- }
(* String in Hinweisfenster bei X,Y anzeigen *)
procedure TStatusWindow.ShowStatus (Pos        : TPoint;
                                    Titel,Stat : string;
                                    AllowBreak : boolean;
                                    AIncrement : integer);
(* Titel  : Fensterberschrift
   Stat   : anzuzeigender String
   AIncrement > 0 : Fortschrittsbalken anzeigen *)
var
  wp   : integer;
  Neu  : boolean;
begin
  Neu:=not Visible;
  Stopped:=false;
  if AllowBreak then begin
    ClientHeight:=btnCancel.Top+btnCancel.Height+5;
    btnCancel.Show;
    end
  else begin
    ClientHeight:=btnCancel.Top;
    btnCancel.Hide;
    end;
  if Neu or (Titel<>'') then Caption:=Titel;
  ProgLabel.Caption:='';
  with StatusLabel do begin
    if Neu then begin
      if Stat<>'' then Caption:=Stat;
      Left:=Rand;
      if Width>Breite then wp:=Left+Width else wp:=Left+Breite;
      end
    else begin
      Caption:=Stat;
      wp:=0;
      end;
    end;
  with BarProgress do begin
    if AIncrement>0 then begin
      Visible:=true; Increment:=AIncrement;
      Status:=psOn;
      end
    else begin
      Visible:=false; Status:=psOff;
      end;
    Left:=Rand; Width:=Breite;
    end;
  if Neu then begin
    BarProgress.Reset;
    ClientWidth:=wp+Rand;
    if Pos.X<0 then Position:=poScreenCenter
    else begin
      Position:=poDesigned;
      with Pos do begin
        if X+Width>Screen.Width then X:=Screen.Width-Width-20;
        if Y+Height>Screen.Height then Y:=Screen.Height-Height-20;
        end;
      Left:=Pos.X; Top:=Pos.Y;
      end;
    btnCancel.Left:=(ClientWidth-btnCancel.Width) div 2;
    end;
  if Neu then Show else Update;
  end;

end.
