(* Delphi Dialog
   Verzeichniswahldialog
   =====================
   - erweiterte Version mit "Hidden" und "System" Verzeichnissen
     (benutzt modifiziertes FileCtrl

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Mrz 2001
   letzte nderung: Okt. 2002
   *)

unit VerzDlg;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Dialogs, FileCtrl, ExtCtrls, SysUtils, IniFiles;

const
  ShowHiddenDirectories : boolean = false;
  ShowSystemDirectories : boolean = false;

type
  TVerzDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Bevel1: TBevel;
    DirectoryListBox: TDirectoryListBox;
    DriveComboBox: TDriveComboBox;
    VerzLabel: TLabel;
    Verzname: TLabel;
    Label1: TLabel;
    UpBtn: TSpeedButton;
    NewBtn: TSpeedButton;
    HomeBtn: TSpeedButton;
    SelectedDir: TComboBox;
    procedure HomeBtnClick(Sender: TObject);
    procedure DirectoryListBoxChange(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure SelectedDirExit(Sender: TObject);
    procedure SelectedDirChange(Sender: TObject);
    procedure SelectedDirKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure VerzDlgShow(Sender: TObject);
    procedure UpBtnClick(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    SDir  : string;
    KSt   : boolean;
    FIniName    : string;
    FIniSection : string;
  protected
    procedure AddItem (ADir : string);
    procedure DeleteItem (ADir : string);
  public
    { Public declarations }
    procedure LoadHistoryList(IniName, Section : string);

    (* Show dialog *)
    function ShowDialog (ATitle,HomeDir : string;
                         var Dir        : string) : boolean;

   (* Dialog an Position anzeigen *)
    function Execute(Titel       : string;
                     StandardDir : string;
                     var Dir     : string;
                     X,Y         : integer) : boolean;
  end;

var
  VerzDialog: TVerzDialog;

implementation

{$R *.DFM}

const
  FMaxLen = 15;

{ ------------------------------------------------------------------- }
procedure TVerzDialog.FormCreate(Sender: TObject);
begin
  FIniName:=''; FIniSection:='Directories';
  DirectoryListBox.ShowHidden:=ShowHiddenDirectories;
  DirectoryListBox.ShowSystem:=ShowSystemDirectories;
  end;

(* load history list *)
procedure TVerzDialog.LoadHistoryList(IniName, Section : string);
var
  i       : integer;
  IniFile : TIniFile;
  s       : string;
begin
  FIniName:=IniName; FIniSection:=Section;
  with SelectedDir do begin
    Items.Clear;
    if (length(FIniName)>0) and (length(FIniSection)>0) then begin
      IniFile:=TIniFile.Create(IniName);
      for i:=0 to FMaxLen-1 do begin
        s:=IniFile.ReadString(FIniSection,'History'+IntToStr(i),'');
        if s<>'' then AddItem(s,nil);
        end;
      IniFile.Free;
      end;
    end;
  end;

(* save history list *)
procedure TVerzDialog.FormDestroy(Sender: TObject);
var
  i       : integer;
  IniFile : TIniFile;
begin
  if (length(FIniName)>0) and (length(FIniSection)>0) then begin
    IniFile:=TIniFile.Create(FIniName);
    IniFile.EraseSection (FIniSection);
    with SelectedDir.Items do for i:=0 to Count-1 do
      IniFile.WriteString(FIniSection,'History'+IntToStr(i),Strings[i]);
    IniFile.Free;
    end;
  end;

{ ------------------------------------------------------------------- }
(* add directory to history list *)
procedure TVerzDialog.AddItem (ADir : string);
begin
  with SelectedDir.Items do begin
    if IndexOf(ADir)<0 then Add (ADir);
    end;
  end;

(* delete directory from history list *)
procedure TVerzDialog.DeleteItem (ADir : string);
var
  i : integer;
begin
  with SelectedDir,Items do begin
    i:=IndexOf(ADir);
    if i>=0 then Delete (i);
    ItemIndex:=0;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Initialize *)
procedure TVerzDialog.VerzDlgShow(Sender: TObject);
begin
  with HomeBtn do begin
    Visible:=SDir<>'';
    Hint:='Home: '+SDir;
    end;
  with SelectedDir do begin
    if Items.Count>0 then Style:=csDropDown else Style:=csSimple;
    SetFocus;
    end;
  end;

(* go to parent directory *)
procedure TVerzDialog.UpBtnClick(Sender: TObject);
var
  s  : string;
begin
  s:=DirectoryListBox.Directory;
  while s[length(s)]<>'\' do delete (s,length(s),1);
  DirectoryListBox.Directory:=s;
  end;

(* create new directory *)
procedure TVerzDialog.NewBtnClick(Sender: TObject);
var
  s : string;

  (* Bei Verzeichnisangaben Backslash ergnzen *)
  function SetDir (Dir : string) : string;
  begin
    if (length(Dir)=0) or (Dir[length(Dir)]='\') then SetDir:=Dir else SetDir:=Dir+'\';
    end;

begin
  s:='';
  if InputQuery (Verzname.Caption,'New directory:',s) then begin
    s:=SetDir(DirectoryListBox.Directory)+s;
    if not ForceDirectories(s) then
        MessageDlg ('Cannot create directory: '+s+'!',mtError,[mbOK],0)
    else begin
//      VerzName.Caption:=s;
      DirectoryListBox.Directory:=s;
//      SelectedDir.Text:=VerzName.Caption;
      end;
    end;
  end;

(* go to home directory *)
procedure TVerzDialog.HomeBtnClick(Sender: TObject);
begin
  if not SetCurrentDir(SDir) then begin
    MessageDlg ('Directory: '+SDir+' not found!',mtError,[mbOK],0);
    DeleteItem(SDir);
    exit;
    end;
//  VerzName.Caption:=SDir;
//  SelectedDir.Text:=VerzName.Caption;
  DirectoryListBox.Directory:=SDir;
  end;

{ ------------------------------------------------------------------- }
procedure TVerzDialog.DirectoryListBoxChange(Sender: TObject);
begin
//  SelectedDir.Text:=DirectoryListBox.Directory;
  SelectedDir.Text:=VerzName.Caption;
  end;

{ ------------------------------------------------------------------- }
procedure TVerzDialog.OKBtnClick(Sender: TObject);
begin
  AddItem(DirectoryListBox.Directory);
  ModalResult:=mrOK;
  end;

procedure TVerzDialog.CancelBtnClick(Sender: TObject);
begin
  ModalResult:=mrCancel;
  end;

{ ------------------------------------------------------------------- }
procedure TVerzDialog.SelectedDirExit(Sender: TObject);
begin
  KSt:=false;
//  VerzName.Caption:=SelectedDir.Text;
  with SelectedDir do DirectoryListBox.Directory:=Items[ItemIndex];
  SelectedDir.Text:=VerzName.Caption;
  end;

procedure TVerzDialog.SelectedDirChange(Sender: TObject);
var
  s : string;
begin
  if not KSt then begin
    with SelectedDir do s:=Items[ItemIndex];
    if not SetCurrentDir(s) then begin
      MessageDlg ('Directory: '+SDir+' not found!',mtError,[mbOK],0);
      DeleteItem(s);
      exit;
      end;
//    VerzName.Caption:=s;
    DirectoryListBox.Directory:=s;
    SelectedDir.Text:=VerzName.Caption;
    end;
  end;

procedure TVerzDialog.SelectedDirKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  KSt:=true;
  end;

{ ------------------------------------------------------------------- }
(* Show dialog *)
function TVerzDialog.ShowDialog (ATitle,HomeDir : string;
                                 var Dir        : string) : boolean;
var
  ok : boolean;
  s  : string;
begin
  Caption:=ATitle;
  SDir:=HomeDir;
  s:=Dir;
  if s='' then s:=GetCurrentDir;
  if not SetCurrentDir(s) then s:=GetCurrentDir;
//  VerzName.Caption:=s;
  DirectoryListBox.Directory:=s;
//  SelectedDir.Text:=VerzName.Caption;
  KSt:=false;
  ok:=ShowModal=mrOK;
  if ok then begin
    Dir:=DirectoryListBox.Directory;
    end;
  Result:=ok;
  end;

{------------------------------------------------------------------- }
(* Dialog an Position anzeigen *)
function TVerzDialog.Execute(Titel       : string;
                             StandardDir : string;
                             var Dir     : string;
                             X,Y         : integer) : boolean;
begin
  if X<0 then Position:=poScreenCenter
  else begin
    if X+Width>Screen.Width then X:=Screen.Width-Width-20;
    if Y+Height>Screen.Height then Y:=Screen.Height-Height-20;
    Left:=X; Top:=Y;
    Position:=poDesigned;
    end;
  Result:=ShowDialog (Titel,StandardDir,Dir);
  end;

end.
